# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_host.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CarlaHostW(object):
    def setupUi(self, CarlaHostW):
        CarlaHostW.setObjectName("CarlaHostW")
        CarlaHostW.resize(1045, 741)
        self.centralwidget = QtWidgets.QWidget(CarlaHostW)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_4.setContentsMargins(0, 4, 0, 1)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.tabWidget.setObjectName("tabWidget")
        self.rack = QtWidgets.QWidget()
        self.rack.setObjectName("rack")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.rack)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 1)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.frame = QtWidgets.QFrame(self.rack)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.frame.setObjectName("frame")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout(self.frame)
        self.horizontalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_10.setSpacing(0)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.pad_left = QtWidgets.QLabel(self.frame)
        self.pad_left.setMinimumSize(QtCore.QSize(25, 0))
        self.pad_left.setMaximumSize(QtCore.QSize(25, 16777215))
        self.pad_left.setText("")
        self.pad_left.setObjectName("pad_left")
        self.horizontalLayout_10.addWidget(self.pad_left)
        self.listWidget = RackListWidget(self.frame)
        self.listWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.listWidget.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.listWidget.setFrameShadow(QtWidgets.QFrame.Plain)
        self.listWidget.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.listWidget.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.listWidget.setObjectName("listWidget")
        self.horizontalLayout_10.addWidget(self.listWidget)
        self.pad_right = QtWidgets.QLabel(self.frame)
        self.pad_right.setMinimumSize(QtCore.QSize(25, 0))
        self.pad_right.setMaximumSize(QtCore.QSize(25, 16777215))
        self.pad_right.setText("")
        self.pad_right.setObjectName("pad_right")
        self.horizontalLayout_10.addWidget(self.pad_right)
        self.rackScrollBar = QtWidgets.QScrollBar(self.frame)
        self.rackScrollBar.setOrientation(QtCore.Qt.Vertical)
        self.rackScrollBar.setObjectName("rackScrollBar")
        self.horizontalLayout_10.addWidget(self.rackScrollBar)
        self.horizontalLayout_2.addWidget(self.frame)
        self.tabWidget.addTab(self.rack, "")
        self.patchbay = QtWidgets.QWidget()
        self.patchbay.setObjectName("patchbay")
        self.gridLayout = QtWidgets.QGridLayout(self.patchbay)
        self.gridLayout.setContentsMargins(0, 0, 0, 1)
        self.gridLayout.setSpacing(1)
        self.gridLayout.setObjectName("gridLayout")
        self.graphicsView = DraggableGraphicsView(self.patchbay)
        self.graphicsView.setTransformationAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)
        self.graphicsView.setObjectName("graphicsView")
        self.gridLayout.addWidget(self.graphicsView, 0, 1, 1, 1)
        self.peak_out = DigitalPeakMeter(self.patchbay)
        self.peak_out.setObjectName("peak_out")
        self.gridLayout.addWidget(self.peak_out, 0, 2, 1, 1)
        self.peak_in = DigitalPeakMeter(self.patchbay)
        self.peak_in.setObjectName("peak_in")
        self.gridLayout.addWidget(self.peak_in, 0, 0, 1, 1)
        self.tabWidget.addTab(self.patchbay, "")
        self.logs = QtWidgets.QWidget()
        self.logs.setObjectName("logs")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.logs)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 1)
        self.horizontalLayout_4.setSpacing(1)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.text_logs = QtWidgets.QPlainTextEdit(self.logs)
        font = QtGui.QFont()
        font.setFamily("DejaVu Sans Mono")
        self.text_logs.setFont(font)
        self.text_logs.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.text_logs.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.text_logs.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.text_logs.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.text_logs.setObjectName("text_logs")
        self.horizontalLayout_4.addWidget(self.text_logs)
        self.log_options = QtWidgets.QWidget(self.logs)
        self.log_options.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.log_options.sizePolicy().hasHeightForWidth())
        self.log_options.setSizePolicy(sizePolicy)
        self.log_options.setMinimumSize(QtCore.QSize(0, 0))
        self.log_options.setObjectName("log_options")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.log_options)
        self.verticalLayout_8.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_8.setContentsMargins(10, 12, 10, -1)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.logs_save = QtWidgets.QPushButton(self.log_options)
        self.logs_save.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.logs_save.setAcceptDrops(False)
        self.logs_save.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.logs_save.setAutoFillBackground(False)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/document-save.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.logs_save.setIcon(icon)
        self.logs_save.setIconSize(QtCore.QSize(16, 16))
        self.logs_save.setAutoDefault(False)
        self.logs_save.setDefault(False)
        self.logs_save.setFlat(False)
        self.logs_save.setObjectName("logs_save")
        self.verticalLayout_8.addWidget(self.logs_save)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem)
        self.logs_clear = QtWidgets.QPushButton(self.log_options)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/16x16/edit-clear.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.logs_clear.setIcon(icon1)
        self.logs_clear.setIconSize(QtCore.QSize(16, 16))
        self.logs_clear.setObjectName("logs_clear")
        self.verticalLayout_8.addWidget(self.logs_clear)
        spacerItem1 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem1)
        self.logs_autoscroll = QtWidgets.QCheckBox(self.log_options)
        self.logs_autoscroll.setChecked(True)
        self.logs_autoscroll.setObjectName("logs_autoscroll")
        self.verticalLayout_8.addWidget(self.logs_autoscroll)
        self.horizontalLayout_4.addWidget(self.log_options, 0, QtCore.Qt.AlignTop)
        self.tabWidget.addTab(self.logs, "")
        self.verticalLayout_4.addWidget(self.tabWidget)
        self.tw_statusbar = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tw_statusbar.sizePolicy().hasHeightForWidth())
        self.tw_statusbar.setSizePolicy(sizePolicy)
        self.tw_statusbar.setObjectName("tw_statusbar")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tw_statusbar)
        self.horizontalLayout_3.setContentsMargins(6, 0, 6, 4)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_buffer_size = QtWidgets.QLabel(self.tw_statusbar)
        self.label_buffer_size.setObjectName("label_buffer_size")
        self.horizontalLayout_3.addWidget(self.label_buffer_size)
        self.cb_buffer_size = QtWidgets.QComboBox(self.tw_statusbar)
        self.cb_buffer_size.setObjectName("cb_buffer_size")
        self.horizontalLayout_3.addWidget(self.cb_buffer_size)
        self.line_1 = QtWidgets.QFrame(self.tw_statusbar)
        self.line_1.setLineWidth(0)
        self.line_1.setMidLineWidth(1)
        self.line_1.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_1.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_1.setObjectName("line_1")
        self.horizontalLayout_3.addWidget(self.line_1)
        self.label_sample_rate = QtWidgets.QLabel(self.tw_statusbar)
        self.label_sample_rate.setObjectName("label_sample_rate")
        self.horizontalLayout_3.addWidget(self.label_sample_rate)
        self.cb_sample_rate = QtWidgets.QComboBox(self.tw_statusbar)
        self.cb_sample_rate.setObjectName("cb_sample_rate")
        self.horizontalLayout_3.addWidget(self.cb_sample_rate)
        self.line_2 = QtWidgets.QFrame(self.tw_statusbar)
        self.line_2.setLineWidth(0)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_3.addWidget(self.line_2)
        self.b_xruns = QtWidgets.QPushButton(self.tw_statusbar)
        self.b_xruns.setFlat(True)
        self.b_xruns.setObjectName("b_xruns")
        self.horizontalLayout_3.addWidget(self.b_xruns)
        self.line_4 = QtWidgets.QFrame(self.tw_statusbar)
        self.line_4.setLineWidth(0)
        self.line_4.setMidLineWidth(1)
        self.line_4.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.horizontalLayout_3.addWidget(self.line_4)
        self.pb_dsp_load = QtWidgets.QProgressBar(self.tw_statusbar)
        self.pb_dsp_load.setProperty("value", 0)
        self.pb_dsp_load.setAlignment(QtCore.Qt.AlignCenter)
        self.pb_dsp_load.setObjectName("pb_dsp_load")
        self.horizontalLayout_3.addWidget(self.pb_dsp_load)
        self.verticalLayout_4.addWidget(self.tw_statusbar)
        CarlaHostW.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(CarlaHostW)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1045, 25))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Engine = QtWidgets.QMenu(self.menubar)
        self.menu_Engine.setObjectName("menu_Engine")
        self.menu_Plugin = QtWidgets.QMenu(self.menubar)
        self.menu_Plugin.setObjectName("menu_Plugin")
        self.menu_PluginMacros = QtWidgets.QMenu(self.menu_Plugin)
        self.menu_PluginMacros.setObjectName("menu_PluginMacros")
        self.menu_Canvas = QtWidgets.QMenu(self.menubar)
        self.menu_Canvas.setObjectName("menu_Canvas")
        self.menu_Canvas_Zoom = QtWidgets.QMenu(self.menu_Canvas)
        self.menu_Canvas_Zoom.setObjectName("menu_Canvas_Zoom")
        self.menu_Settings = QtWidgets.QMenu(self.menubar)
        self.menu_Settings.setObjectName("menu_Settings")
        self.menu_Help = QtWidgets.QMenu(self.menubar)
        self.menu_Help.setObjectName("menu_Help")
        CarlaHostW.setMenuBar(self.menubar)
        self.toolBar = QtWidgets.QToolBar(CarlaHostW)
        self.toolBar.setMovable(False)
        self.toolBar.setAllowedAreas(QtCore.Qt.NoToolBarArea)
        self.toolBar.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.toolBar.setObjectName("toolBar")
        CarlaHostW.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.dockWidget = QtWidgets.QDockWidget(CarlaHostW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(10)
        sizePolicy.setHeightForWidth(self.dockWidget.sizePolicy().hasHeightForWidth())
        self.dockWidget.setSizePolicy(sizePolicy)
        self.dockWidget.setFeatures(QtWidgets.QDockWidget.NoDockWidgetFeatures)
        self.dockWidget.setAllowedAreas(QtCore.Qt.NoDockWidgetArea)
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setContentsMargins(1, 5, 0, 1)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(self.dockWidgetContents)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.tabUtils = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(10)
        sizePolicy.setHeightForWidth(self.tabUtils.sizePolicy().hasHeightForWidth())
        self.tabUtils.setSizePolicy(sizePolicy)
        self.tabUtils.setMinimumSize(QtCore.QSize(210, 0))
        self.tabUtils.setAcceptDrops(False)
        self.tabUtils.setTabPosition(QtWidgets.QTabWidget.West)
        self.tabUtils.setObjectName("tabUtils")
        self.disk = QtWidgets.QWidget()
        self.disk.setAcceptDrops(False)
        self.disk.setObjectName("disk")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.disk)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 1)
        self.verticalLayout_3.setSpacing(1)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.cb_disk = QtWidgets.QComboBox(self.disk)
        self.cb_disk.setObjectName("cb_disk")
        self.cb_disk.addItem("")
        self.horizontalLayout.addWidget(self.cb_disk)
        self.b_disk_add = QtWidgets.QToolButton(self.disk)
        self.b_disk_add.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/16x16/list-add.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_disk_add.setIcon(icon2)
        self.b_disk_add.setObjectName("b_disk_add")
        self.horizontalLayout.addWidget(self.b_disk_add)
        self.b_disk_remove = QtWidgets.QToolButton(self.disk)
        self.b_disk_remove.setEnabled(False)
        self.b_disk_remove.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/16x16/list-remove.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_disk_remove.setIcon(icon3)
        self.b_disk_remove.setObjectName("b_disk_remove")
        self.horizontalLayout.addWidget(self.b_disk_remove)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.fileTreeView = QtWidgets.QTreeView(self.disk)
        self.fileTreeView.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.fileTreeView.setDragEnabled(True)
        self.fileTreeView.setDragDropMode(QtWidgets.QAbstractItemView.DragOnly)
        self.fileTreeView.setObjectName("fileTreeView")
        self.verticalLayout_3.addWidget(self.fileTreeView)
        self.tabUtils.addTab(self.disk, "")
        self.w_transport = QtWidgets.QWidget()
        self.w_transport.setObjectName("w_transport")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.w_transport)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.group_transport_controls = QtWidgets.QGroupBox(self.w_transport)
        self.group_transport_controls.setObjectName("group_transport_controls")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.group_transport_controls)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.b_transport_play = QtWidgets.QPushButton(self.group_transport_controls)
        self.b_transport_play.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/16x16/media-playback-start.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_transport_play.setIcon(icon4)
        self.b_transport_play.setCheckable(True)
        self.b_transport_play.setObjectName("b_transport_play")
        self.horizontalLayout_5.addWidget(self.b_transport_play)
        self.b_transport_stop = QtWidgets.QPushButton(self.group_transport_controls)
        self.b_transport_stop.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/16x16/media-playback-stop.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_transport_stop.setIcon(icon5)
        self.b_transport_stop.setObjectName("b_transport_stop")
        self.horizontalLayout_5.addWidget(self.b_transport_stop)
        self.b_transport_backwards = QtWidgets.QPushButton(self.group_transport_controls)
        self.b_transport_backwards.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/16x16/media-seek-backward.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_transport_backwards.setIcon(icon6)
        self.b_transport_backwards.setAutoRepeat(True)
        self.b_transport_backwards.setObjectName("b_transport_backwards")
        self.horizontalLayout_5.addWidget(self.b_transport_backwards)
        self.b_transport_forwards = QtWidgets.QPushButton(self.group_transport_controls)
        self.b_transport_forwards.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/16x16/media-seek-forward.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_transport_forwards.setIcon(icon7)
        self.b_transport_forwards.setAutoRepeat(True)
        self.b_transport_forwards.setObjectName("b_transport_forwards")
        self.horizontalLayout_5.addWidget(self.b_transport_forwards)
        self.verticalLayout_6.addWidget(self.group_transport_controls)
        self.group_transport_info = QtWidgets.QGroupBox(self.w_transport)
        self.group_transport_info.setObjectName("group_transport_info")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.group_transport_info)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_transport_frame = QtWidgets.QLabel(self.group_transport_info)
        self.label_transport_frame.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_transport_frame.setObjectName("label_transport_frame")
        self.horizontalLayout_6.addWidget(self.label_transport_frame)
        self.l_transport_frame = QtWidgets.QLabel(self.group_transport_info)
        font = QtGui.QFont()
        font.setFamily("DejaVu Sans Mono")
        self.l_transport_frame.setFont(font)
        self.l_transport_frame.setObjectName("l_transport_frame")
        self.horizontalLayout_6.addWidget(self.l_transport_frame)
        spacerItem2 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_transport_time = QtWidgets.QLabel(self.group_transport_info)
        self.label_transport_time.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_transport_time.setObjectName("label_transport_time")
        self.horizontalLayout_7.addWidget(self.label_transport_time)
        self.l_transport_time = QtWidgets.QLabel(self.group_transport_info)
        font = QtGui.QFont()
        font.setFamily("DejaVu Sans Mono")
        self.l_transport_time.setFont(font)
        self.l_transport_time.setObjectName("l_transport_time")
        self.horizontalLayout_7.addWidget(self.l_transport_time)
        spacerItem3 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_transport_bbt = QtWidgets.QLabel(self.group_transport_info)
        self.label_transport_bbt.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_transport_bbt.setObjectName("label_transport_bbt")
        self.horizontalLayout_8.addWidget(self.label_transport_bbt)
        self.l_transport_bbt = QtWidgets.QLabel(self.group_transport_info)
        font = QtGui.QFont()
        font.setFamily("DejaVu Sans Mono")
        self.l_transport_bbt.setFont(font)
        self.l_transport_bbt.setObjectName("l_transport_bbt")
        self.horizontalLayout_8.addWidget(self.l_transport_bbt)
        spacerItem4 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        self.verticalLayout_6.addWidget(self.group_transport_info)
        self.group_transport_settings = QtWidgets.QGroupBox(self.w_transport)
        self.group_transport_settings.setObjectName("group_transport_settings")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.group_transport_settings)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.dsb_transport_bpm = QtWidgets.QDoubleSpinBox(self.group_transport_settings)
        self.dsb_transport_bpm.setDecimals(1)
        self.dsb_transport_bpm.setMinimum(20.0)
        self.dsb_transport_bpm.setMaximum(999.0)
        self.dsb_transport_bpm.setObjectName("dsb_transport_bpm")
        self.verticalLayout_5.addWidget(self.dsb_transport_bpm)
        self.cb_transport_jack = QtWidgets.QCheckBox(self.group_transport_settings)
        self.cb_transport_jack.setObjectName("cb_transport_jack")
        self.verticalLayout_5.addWidget(self.cb_transport_jack)
        self.cb_transport_link = QtWidgets.QCheckBox(self.group_transport_settings)
        self.cb_transport_link.setObjectName("cb_transport_link")
        self.verticalLayout_5.addWidget(self.cb_transport_link)
        self.verticalLayout_6.addWidget(self.group_transport_settings)
        spacerItem5 = QtWidgets.QSpacerItem(20, 188, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem5)
        self.tabUtils.addTab(self.w_transport, "")
        self.tw_miniCanvas = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tw_miniCanvas.sizePolicy().hasHeightForWidth())
        self.tw_miniCanvas.setSizePolicy(sizePolicy)
        self.tw_miniCanvas.setTabPosition(QtWidgets.QTabWidget.West)
        self.tw_miniCanvas.setProperty("tabBarAutoHide", False)
        self.tw_miniCanvas.setObjectName("tw_miniCanvas")
        self.tab_3 = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tab_3.sizePolicy().hasHeightForWidth())
        self.tab_3.setSizePolicy(sizePolicy)
        self.tab_3.setObjectName("tab_3")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.tab_3)
        self.verticalLayout_7.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 1)
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.miniCanvasPreview = CanvasPreviewFrame(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.miniCanvasPreview.sizePolicy().hasHeightForWidth())
        self.miniCanvasPreview.setSizePolicy(sizePolicy)
        self.miniCanvasPreview.setMinimumSize(QtCore.QSize(210, 162))
        self.miniCanvasPreview.setAcceptDrops(False)
        self.miniCanvasPreview.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.miniCanvasPreview.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.miniCanvasPreview.setLineWidth(1)
        self.miniCanvasPreview.setObjectName("miniCanvasPreview")
        self.verticalLayout_7.addWidget(self.miniCanvasPreview)
        self.tw_miniCanvas.addTab(self.tab_3, "")
        self.verticalLayout.addWidget(self.splitter)
        self.dockWidget.setWidget(self.dockWidgetContents)
        CarlaHostW.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget)
        self.act_file_new = QtWidgets.QAction(CarlaHostW)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/16x16/document-new.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_new.setIcon(icon8)
        self.act_file_new.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_new.setObjectName("act_file_new")
        self.act_file_open = QtWidgets.QAction(CarlaHostW)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/16x16/document-open.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_open.setIcon(icon9)
        self.act_file_open.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_open.setObjectName("act_file_open")
        self.act_file_save = QtWidgets.QAction(CarlaHostW)
        self.act_file_save.setIcon(icon)
        self.act_file_save.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_save.setObjectName("act_file_save")
        self.act_file_save_as = QtWidgets.QAction(CarlaHostW)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/16x16/document-save-as.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_save_as.setIcon(icon10)
        self.act_file_save_as.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_save_as.setObjectName("act_file_save_as")
        self.act_file_quit = QtWidgets.QAction(CarlaHostW)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/16x16/application-exit.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_quit.setIcon(icon11)
        self.act_file_quit.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_quit.setObjectName("act_file_quit")
        self.act_engine_start = QtWidgets.QAction(CarlaHostW)
        self.act_engine_start.setIcon(icon4)
        self.act_engine_start.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_engine_start.setObjectName("act_engine_start")
        self.act_engine_stop = QtWidgets.QAction(CarlaHostW)
        self.act_engine_stop.setIcon(icon5)
        self.act_engine_stop.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_engine_stop.setObjectName("act_engine_stop")
        self.act_plugin_add = QtWidgets.QAction(CarlaHostW)
        self.act_plugin_add.setIcon(icon2)
        self.act_plugin_add.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugin_add.setObjectName("act_plugin_add")
        self.act_plugin_remove_all = QtWidgets.QAction(CarlaHostW)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/16x16/edit-delete.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_plugin_remove_all.setIcon(icon12)
        self.act_plugin_remove_all.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugin_remove_all.setObjectName("act_plugin_remove_all")
        self.act_plugins_enable = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_enable.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_enable.setObjectName("act_plugins_enable")
        self.act_plugins_disable = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_disable.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_disable.setObjectName("act_plugins_disable")
        self.act_plugins_bypass = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_bypass.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_bypass.setObjectName("act_plugins_bypass")
        self.act_plugins_wet100 = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_wet100.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_wet100.setObjectName("act_plugins_wet100")
        self.act_plugins_mute = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_mute.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_mute.setObjectName("act_plugins_mute")
        self.act_plugins_volume100 = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_volume100.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_volume100.setObjectName("act_plugins_volume100")
        self.act_plugins_center = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_center.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_center.setObjectName("act_plugins_center")
        self.act_transport_play = QtWidgets.QAction(CarlaHostW)
        self.act_transport_play.setCheckable(True)
        self.act_transport_play.setIcon(icon4)
        self.act_transport_play.setObjectName("act_transport_play")
        self.act_transport_stop = QtWidgets.QAction(CarlaHostW)
        self.act_transport_stop.setIcon(icon5)
        self.act_transport_stop.setObjectName("act_transport_stop")
        self.act_transport_backwards = QtWidgets.QAction(CarlaHostW)
        self.act_transport_backwards.setIcon(icon6)
        self.act_transport_backwards.setObjectName("act_transport_backwards")
        self.act_transport_forwards = QtWidgets.QAction(CarlaHostW)
        self.act_transport_forwards.setIcon(icon7)
        self.act_transport_forwards.setObjectName("act_transport_forwards")
        self.act_canvas_arrange = QtWidgets.QAction(CarlaHostW)
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/16x16/view-sort-ascending.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_arrange.setIcon(icon13)
        self.act_canvas_arrange.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_arrange.setObjectName("act_canvas_arrange")
        self.act_canvas_refresh = QtWidgets.QAction(CarlaHostW)
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(":/16x16/view-refresh.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_refresh.setIcon(icon14)
        self.act_canvas_refresh.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_refresh.setObjectName("act_canvas_refresh")
        self.act_canvas_save_image = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_save_image.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_save_image.setObjectName("act_canvas_save_image")
        self.act_canvas_zoom_fit = QtWidgets.QAction(CarlaHostW)
        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap(":/16x16/zoom-fit-best.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_zoom_fit.setIcon(icon15)
        self.act_canvas_zoom_fit.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_zoom_fit.setObjectName("act_canvas_zoom_fit")
        self.act_canvas_zoom_in = QtWidgets.QAction(CarlaHostW)
        icon16 = QtGui.QIcon()
        icon16.addPixmap(QtGui.QPixmap(":/16x16/zoom-in.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_zoom_in.setIcon(icon16)
        self.act_canvas_zoom_in.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_zoom_in.setObjectName("act_canvas_zoom_in")
        self.act_canvas_zoom_out = QtWidgets.QAction(CarlaHostW)
        icon17 = QtGui.QIcon()
        icon17.addPixmap(QtGui.QPixmap(":/16x16/zoom-out.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_zoom_out.setIcon(icon17)
        self.act_canvas_zoom_out.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_zoom_out.setObjectName("act_canvas_zoom_out")
        self.act_canvas_zoom_100 = QtWidgets.QAction(CarlaHostW)
        icon18 = QtGui.QIcon()
        icon18.addPixmap(QtGui.QPixmap(":/16x16/zoom-original.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_zoom_100.setIcon(icon18)
        self.act_canvas_zoom_100.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_zoom_100.setObjectName("act_canvas_zoom_100")
        self.act_settings_show_toolbar = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_toolbar.setCheckable(True)
        self.act_settings_show_toolbar.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_show_toolbar.setObjectName("act_settings_show_toolbar")
        self.act_settings_configure = QtWidgets.QAction(CarlaHostW)
        icon19 = QtGui.QIcon()
        icon19.addPixmap(QtGui.QPixmap(":/16x16/configure.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_settings_configure.setIcon(icon19)
        self.act_settings_configure.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_configure.setObjectName("act_settings_configure")
        self.act_help_about = QtWidgets.QAction(CarlaHostW)
        self.act_help_about.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_help_about.setObjectName("act_help_about")
        self.act_help_about_juce = QtWidgets.QAction(CarlaHostW)
        self.act_help_about_juce.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_help_about_juce.setObjectName("act_help_about_juce")
        self.act_help_about_qt = QtWidgets.QAction(CarlaHostW)
        self.act_help_about_qt.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_help_about_qt.setObjectName("act_help_about_qt")
        self.act_settings_show_meters = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_meters.setCheckable(True)
        self.act_settings_show_meters.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_show_meters.setObjectName("act_settings_show_meters")
        self.act_settings_show_keyboard = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_keyboard.setCheckable(True)
        self.act_settings_show_keyboard.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_show_keyboard.setObjectName("act_settings_show_keyboard")
        self.act_canvas_show_internal = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_show_internal.setCheckable(True)
        self.act_canvas_show_internal.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_show_internal.setObjectName("act_canvas_show_internal")
        self.act_canvas_show_external = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_show_external.setCheckable(True)
        self.act_canvas_show_external.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_show_external.setObjectName("act_canvas_show_external")
        self.act_settings_show_time_panel = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_time_panel.setCheckable(True)
        self.act_settings_show_time_panel.setObjectName("act_settings_show_time_panel")
        self.act_settings_show_side_panel = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_side_panel.setCheckable(True)
        self.act_settings_show_side_panel.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_show_side_panel.setObjectName("act_settings_show_side_panel")
        self.act_file_connect = QtWidgets.QAction(CarlaHostW)
        icon20 = QtGui.QIcon()
        icon20.addPixmap(QtGui.QPixmap(":/16x16/network-connect.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_connect.setIcon(icon20)
        self.act_file_connect.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_connect.setObjectName("act_file_connect")
        self.act_file_refresh = QtWidgets.QAction(CarlaHostW)
        self.act_file_refresh.setIcon(icon14)
        self.act_file_refresh.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_refresh.setObjectName("act_file_refresh")
        self.act_plugins_compact = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_compact.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_compact.setObjectName("act_plugins_compact")
        self.act_plugins_expand = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_expand.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_expand.setObjectName("act_plugins_expand")
        self.act_secret_1 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_1.setObjectName("act_secret_1")
        self.act_secret_2 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_2.setObjectName("act_secret_2")
        self.act_secret_3 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_3.setObjectName("act_secret_3")
        self.act_secret_4 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_4.setObjectName("act_secret_4")
        self.act_secret_5 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_5.setObjectName("act_secret_5")
        self.act_plugin_add_jack = QtWidgets.QAction(CarlaHostW)
        self.act_plugin_add_jack.setIcon(icon2)
        self.act_plugin_add_jack.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugin_add_jack.setObjectName("act_plugin_add_jack")
        self.act_engine_config = QtWidgets.QAction(CarlaHostW)
        self.act_engine_config.setIcon(icon19)
        self.act_engine_config.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_engine_config.setObjectName("act_engine_config")
        self.act_engine_panic = QtWidgets.QAction(CarlaHostW)
        icon21 = QtGui.QIcon()
        icon21.addPixmap(QtGui.QPixmap(":/16x16/dialog-warning.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_engine_panic.setIcon(icon21)
        self.act_engine_panic.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_engine_panic.setObjectName("act_engine_panic")
        self.act_engine_panel = QtWidgets.QAction(CarlaHostW)
        self.act_engine_panel.setObjectName("act_engine_panel")
        self.act_canvas_save_image_2x = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_save_image_2x.setObjectName("act_canvas_save_image_2x")
        self.act_canvas_save_image_4x = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_save_image_4x.setObjectName("act_canvas_save_image_4x")
        self.act_canvas_copy_clipboard = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_copy_clipboard.setObjectName("act_canvas_copy_clipboard")
        self.menu_File.addAction(self.act_file_connect)
        self.menu_File.addAction(self.act_file_refresh)
        self.menu_File.addAction(self.act_file_new)
        self.menu_File.addAction(self.act_file_open)
        self.menu_File.addAction(self.act_file_save)
        self.menu_File.addAction(self.act_file_save_as)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.act_file_quit)
        self.menu_Engine.addAction(self.act_engine_start)
        self.menu_Engine.addAction(self.act_engine_stop)
        self.menu_Engine.addAction(self.act_engine_panic)
        self.menu_Engine.addSeparator()
        self.menu_Engine.addAction(self.act_engine_config)
        self.menu_PluginMacros.addAction(self.act_plugins_enable)
        self.menu_PluginMacros.addAction(self.act_plugins_disable)
        self.menu_PluginMacros.addSeparator()
        self.menu_PluginMacros.addAction(self.act_plugins_volume100)
        self.menu_PluginMacros.addAction(self.act_plugins_mute)
        self.menu_PluginMacros.addSeparator()
        self.menu_PluginMacros.addAction(self.act_plugins_wet100)
        self.menu_PluginMacros.addAction(self.act_plugins_bypass)
        self.menu_PluginMacros.addSeparator()
        self.menu_PluginMacros.addAction(self.act_plugins_center)
        self.menu_PluginMacros.addSeparator()
        self.menu_PluginMacros.addAction(self.act_plugins_compact)
        self.menu_PluginMacros.addAction(self.act_plugins_expand)
        self.menu_Plugin.addAction(self.act_plugin_add)
        self.menu_Plugin.addAction(self.act_plugin_add_jack)
        self.menu_Plugin.addAction(self.act_plugin_remove_all)
        self.menu_Plugin.addSeparator()
        self.menu_Plugin.addAction(self.menu_PluginMacros.menuAction())
        self.menu_Canvas_Zoom.addAction(self.act_canvas_zoom_fit)
        self.menu_Canvas_Zoom.addSeparator()
        self.menu_Canvas_Zoom.addAction(self.act_canvas_zoom_in)
        self.menu_Canvas_Zoom.addAction(self.act_canvas_zoom_out)
        self.menu_Canvas_Zoom.addAction(self.act_canvas_zoom_100)
        self.menu_Canvas.addAction(self.act_canvas_show_internal)
        self.menu_Canvas.addAction(self.act_canvas_show_external)
        self.menu_Canvas.addSeparator()
        self.menu_Canvas.addAction(self.act_canvas_arrange)
        self.menu_Canvas.addAction(self.act_canvas_refresh)
        self.menu_Canvas.addAction(self.menu_Canvas_Zoom.menuAction())
        self.menu_Canvas.addSeparator()
        self.menu_Canvas.addAction(self.act_canvas_save_image)
        self.menu_Canvas.addAction(self.act_canvas_save_image_2x)
        self.menu_Canvas.addAction(self.act_canvas_save_image_4x)
        self.menu_Canvas.addAction(self.act_canvas_copy_clipboard)
        self.menu_Settings.addAction(self.act_settings_show_toolbar)
        self.menu_Settings.addAction(self.act_settings_show_meters)
        self.menu_Settings.addAction(self.act_settings_show_keyboard)
        self.menu_Settings.addAction(self.act_settings_show_side_panel)
        self.menu_Settings.addSeparator()
        self.menu_Settings.addAction(self.act_settings_configure)
        self.menu_Help.addAction(self.act_help_about)
        self.menu_Help.addAction(self.act_help_about_juce)
        self.menu_Help.addAction(self.act_help_about_qt)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Engine.menuAction())
        self.menubar.addAction(self.menu_Plugin.menuAction())
        self.menubar.addAction(self.menu_Canvas.menuAction())
        self.menubar.addAction(self.menu_Settings.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())
        self.toolBar.addAction(self.act_file_new)
        self.toolBar.addAction(self.act_file_open)
        self.toolBar.addAction(self.act_file_save)
        self.toolBar.addAction(self.act_file_save_as)
        self.toolBar.addAction(self.act_file_connect)
        self.toolBar.addAction(self.act_file_refresh)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.act_plugin_add)
        self.toolBar.addAction(self.act_plugin_add_jack)
        self.toolBar.addAction(self.act_plugin_remove_all)
        self.toolBar.addAction(self.act_engine_panic)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.act_settings_configure)

        self.retranslateUi(CarlaHostW)
        self.tabWidget.setCurrentIndex(0)
        self.tabUtils.setCurrentIndex(0)
        self.tw_miniCanvas.setCurrentIndex(0)
        self.act_file_quit.triggered.connect(CarlaHostW.close) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(CarlaHostW)

    def retranslateUi(self, CarlaHostW):
        _translate = QtCore.QCoreApplication.translate
        CarlaHostW.setWindowTitle(_translate("CarlaHostW", "MainWindow"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.rack), _translate("CarlaHostW", "Rack"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.patchbay), _translate("CarlaHostW", "Patchbay"))
        self.text_logs.setPlainText(_translate("CarlaHostW", "Loading..."))
        self.logs_save.setText(_translate("CarlaHostW", " Save"))
        self.logs_clear.setText(_translate("CarlaHostW", " Clear"))
        self.logs_clear.setShortcut(_translate("CarlaHostW", "Ctrl+L"))
        self.logs_autoscroll.setText(_translate("CarlaHostW", "Auto-Scroll"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.logs), _translate("CarlaHostW", "Logs"))
        self.label_buffer_size.setText(_translate("CarlaHostW", "Buffer Size:"))
        self.label_sample_rate.setText(_translate("CarlaHostW", "Sample Rate:"))
        self.b_xruns.setText(_translate("CarlaHostW", "? Xruns"))
        self.pb_dsp_load.setFormat(_translate("CarlaHostW", "DSP Load: %p%"))
        self.menu_File.setTitle(_translate("CarlaHostW", "&File"))
        self.menu_Engine.setTitle(_translate("CarlaHostW", "&Engine"))
        self.menu_Plugin.setTitle(_translate("CarlaHostW", "&Plugin"))
        self.menu_PluginMacros.setTitle(_translate("CarlaHostW", "Macros (all plugins)"))
        self.menu_Canvas.setTitle(_translate("CarlaHostW", "&Canvas"))
        self.menu_Canvas_Zoom.setTitle(_translate("CarlaHostW", "Zoom"))
        self.menu_Settings.setTitle(_translate("CarlaHostW", "&Settings"))
        self.menu_Help.setTitle(_translate("CarlaHostW", "&Help"))
        self.toolBar.setWindowTitle(_translate("CarlaHostW", "Tool Bar"))
        self.cb_disk.setItemText(0, _translate("CarlaHostW", "Home"))
        self.tabUtils.setTabText(self.tabUtils.indexOf(self.disk), _translate("CarlaHostW", "Disk"))
        self.group_transport_controls.setTitle(_translate("CarlaHostW", "Playback Controls"))
        self.group_transport_info.setTitle(_translate("CarlaHostW", "Time Information"))
        self.label_transport_frame.setText(_translate("CarlaHostW", "Frame:"))
        self.l_transport_frame.setText(_translate("CarlaHostW", "000\'000\'000"))
        self.label_transport_time.setText(_translate("CarlaHostW", "Time:"))
        self.l_transport_time.setText(_translate("CarlaHostW", "00:00:00"))
        self.label_transport_bbt.setText(_translate("CarlaHostW", "BBT:"))
        self.l_transport_bbt.setText(_translate("CarlaHostW", "000|00|0000"))
        self.group_transport_settings.setTitle(_translate("CarlaHostW", "Settings"))
        self.dsb_transport_bpm.setSuffix(_translate("CarlaHostW", " BPM"))
        self.cb_transport_jack.setText(_translate("CarlaHostW", "Use JACK Transport"))
        self.cb_transport_link.setText(_translate("CarlaHostW", "Use Ableton Link"))
        self.tabUtils.setTabText(self.tabUtils.indexOf(self.w_transport), _translate("CarlaHostW", "Transport"))
        self.act_file_new.setText(_translate("CarlaHostW", "&New"))
        self.act_file_new.setShortcut(_translate("CarlaHostW", "Ctrl+N"))
        self.act_file_open.setText(_translate("CarlaHostW", "&Open..."))
        self.act_file_open.setIconText(_translate("CarlaHostW", "Open..."))
        self.act_file_open.setToolTip(_translate("CarlaHostW", "Open..."))
        self.act_file_open.setShortcut(_translate("CarlaHostW", "Ctrl+O"))
        self.act_file_save.setText(_translate("CarlaHostW", "&Save"))
        self.act_file_save.setShortcut(_translate("CarlaHostW", "Ctrl+S"))
        self.act_file_save_as.setText(_translate("CarlaHostW", "Save &As..."))
        self.act_file_save_as.setIconText(_translate("CarlaHostW", "Save As..."))
        self.act_file_save_as.setToolTip(_translate("CarlaHostW", "Save As..."))
        self.act_file_save_as.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+S"))
        self.act_file_quit.setText(_translate("CarlaHostW", "&Quit"))
        self.act_file_quit.setShortcut(_translate("CarlaHostW", "Ctrl+Q"))
        self.act_engine_start.setText(_translate("CarlaHostW", "&Start"))
        self.act_engine_start.setShortcut(_translate("CarlaHostW", "F5"))
        self.act_engine_stop.setText(_translate("CarlaHostW", "St&op"))
        self.act_engine_stop.setShortcut(_translate("CarlaHostW", "F6"))
        self.act_plugin_add.setText(_translate("CarlaHostW", "&Add Plugin..."))
        self.act_plugin_add.setShortcut(_translate("CarlaHostW", "Ctrl+A"))
        self.act_plugin_remove_all.setText(_translate("CarlaHostW", "&Remove All"))
        self.act_plugins_enable.setText(_translate("CarlaHostW", "Enable"))
        self.act_plugins_disable.setText(_translate("CarlaHostW", "Disable"))
        self.act_plugins_bypass.setText(_translate("CarlaHostW", "0% Wet (Bypass)"))
        self.act_plugins_wet100.setText(_translate("CarlaHostW", "100% Wet"))
        self.act_plugins_mute.setText(_translate("CarlaHostW", "0% Volume (Mute)"))
        self.act_plugins_volume100.setText(_translate("CarlaHostW", "100% Volume"))
        self.act_plugins_center.setText(_translate("CarlaHostW", "Center Balance"))
        self.act_transport_play.setText(_translate("CarlaHostW", "&Play"))
        self.act_transport_play.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+P"))
        self.act_transport_stop.setText(_translate("CarlaHostW", "&Stop"))
        self.act_transport_stop.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+X"))
        self.act_transport_backwards.setText(_translate("CarlaHostW", "&Backwards"))
        self.act_transport_backwards.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+B"))
        self.act_transport_forwards.setText(_translate("CarlaHostW", "&Forwards"))
        self.act_transport_forwards.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+F"))
        self.act_canvas_arrange.setText(_translate("CarlaHostW", "&Arrange"))
        self.act_canvas_arrange.setShortcut(_translate("CarlaHostW", "Ctrl+G"))
        self.act_canvas_refresh.setText(_translate("CarlaHostW", "&Refresh"))
        self.act_canvas_refresh.setShortcut(_translate("CarlaHostW", "Ctrl+R"))
        self.act_canvas_save_image.setText(_translate("CarlaHostW", "Save &Image..."))
        self.act_canvas_zoom_fit.setText(_translate("CarlaHostW", "Auto-Fit"))
        self.act_canvas_zoom_fit.setShortcut(_translate("CarlaHostW", "Home"))
        self.act_canvas_zoom_in.setText(_translate("CarlaHostW", "Zoom In"))
        self.act_canvas_zoom_in.setShortcut(_translate("CarlaHostW", "Ctrl++"))
        self.act_canvas_zoom_out.setText(_translate("CarlaHostW", "Zoom Out"))
        self.act_canvas_zoom_out.setShortcut(_translate("CarlaHostW", "Ctrl+-"))
        self.act_canvas_zoom_100.setText(_translate("CarlaHostW", "Zoom 100%"))
        self.act_canvas_zoom_100.setShortcut(_translate("CarlaHostW", "Ctrl+1"))
        self.act_settings_show_toolbar.setText(_translate("CarlaHostW", "Show &Toolbar"))
        self.act_settings_configure.setText(_translate("CarlaHostW", "&Configure Carla"))
        self.act_help_about.setText(_translate("CarlaHostW", "&About"))
        self.act_help_about_juce.setText(_translate("CarlaHostW", "About &JUCE"))
        self.act_help_about_qt.setText(_translate("CarlaHostW", "About &Qt"))
        self.act_settings_show_meters.setText(_translate("CarlaHostW", "Show Canvas &Meters"))
        self.act_settings_show_keyboard.setText(_translate("CarlaHostW", "Show Canvas &Keyboard"))
        self.act_canvas_show_internal.setText(_translate("CarlaHostW", "Show Internal"))
        self.act_canvas_show_external.setText(_translate("CarlaHostW", "Show External"))
        self.act_settings_show_time_panel.setText(_translate("CarlaHostW", "Show Time Panel"))
        self.act_settings_show_side_panel.setText(_translate("CarlaHostW", "Show &Side Panel"))
        self.act_settings_show_side_panel.setShortcut(_translate("CarlaHostW", "Ctrl+P"))
        self.act_file_connect.setText(_translate("CarlaHostW", "&Connect..."))
        self.act_file_refresh.setText(_translate("CarlaHostW", "&Refresh"))
        self.act_plugins_compact.setText(_translate("CarlaHostW", "Compact Slots"))
        self.act_plugins_expand.setText(_translate("CarlaHostW", "Expand Slots"))
        self.act_secret_1.setText(_translate("CarlaHostW", "Perform secret 1"))
        self.act_secret_2.setText(_translate("CarlaHostW", "Perform secret 2"))
        self.act_secret_3.setText(_translate("CarlaHostW", "Perform secret 3"))
        self.act_secret_4.setText(_translate("CarlaHostW", "Perform secret 4"))
        self.act_secret_5.setText(_translate("CarlaHostW", "Perform secret 5"))
        self.act_plugin_add_jack.setText(_translate("CarlaHostW", "Add &JACK Application..."))
        self.act_engine_config.setText(_translate("CarlaHostW", "&Configure driver..."))
        self.act_engine_panic.setText(_translate("CarlaHostW", "Panic"))
        self.act_engine_panel.setText(_translate("CarlaHostW", "Open custom driver panel..."))
        self.act_canvas_save_image_2x.setText(_translate("CarlaHostW", "Save Image... (2x zoom)"))
        self.act_canvas_save_image_4x.setText(_translate("CarlaHostW", "Save Image... (4x zoom)"))
        self.act_canvas_copy_clipboard.setText(_translate("CarlaHostW", "Copy as Image to Clipboard"))
        self.act_canvas_copy_clipboard.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+C"))
from widgets.canvaspreviewframe import CanvasPreviewFrame
from widgets.digitalpeakmeter import DigitalPeakMeter
from widgets.draggablegraphicsview import DraggableGraphicsView
from widgets.racklistwidget import RackListWidget
import resources_rc
