# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_refresh.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginRefreshW(object):
    def setupUi(self, PluginRefreshW):
        PluginRefreshW.setObjectName("PluginRefreshW")
        PluginRefreshW.resize(613, 369)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(PluginRefreshW)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.group_types = QtWidgets.QGroupBox(PluginRefreshW)
        self.group_types.setAlignment(QtCore.Qt.AlignCenter)
        self.group_types.setObjectName("group_types")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.group_types)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.ch_ladspa = QtWidgets.QCheckBox(self.group_types)
        self.ch_ladspa.setObjectName("ch_ladspa")
        self.verticalLayout.addWidget(self.ch_ladspa)
        self.ch_dssi = QtWidgets.QCheckBox(self.group_types)
        self.ch_dssi.setObjectName("ch_dssi")
        self.verticalLayout.addWidget(self.ch_dssi)
        self.ch_lv2 = QtWidgets.QCheckBox(self.group_types)
        self.ch_lv2.setObjectName("ch_lv2")
        self.verticalLayout.addWidget(self.ch_lv2)
        self.ch_vst = QtWidgets.QCheckBox(self.group_types)
        self.ch_vst.setObjectName("ch_vst")
        self.verticalLayout.addWidget(self.ch_vst)
        self.ch_vst3 = QtWidgets.QCheckBox(self.group_types)
        self.ch_vst3.setObjectName("ch_vst3")
        self.verticalLayout.addWidget(self.ch_vst3)
        self.ch_au = QtWidgets.QCheckBox(self.group_types)
        self.ch_au.setObjectName("ch_au")
        self.verticalLayout.addWidget(self.ch_au)
        self.ch_jsfx = QtWidgets.QCheckBox(self.group_types)
        self.ch_jsfx.setObjectName("ch_jsfx")
        self.verticalLayout.addWidget(self.ch_jsfx)
        self.line_3 = QtWidgets.QFrame(self.group_types)
        self.line_3.setLineWidth(0)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout.addWidget(self.line_3)
        self.ch_sf2 = QtWidgets.QCheckBox(self.group_types)
        self.ch_sf2.setObjectName("ch_sf2")
        self.verticalLayout.addWidget(self.ch_sf2)
        self.ch_sfz = QtWidgets.QCheckBox(self.group_types)
        self.ch_sfz.setObjectName("ch_sfz")
        self.verticalLayout.addWidget(self.ch_sfz)
        spacerItem1 = QtWidgets.QSpacerItem(40, 5, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout.addItem(spacerItem1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.sep_format = QtWidgets.QFrame(self.group_types)
        self.sep_format.setLineWidth(0)
        self.sep_format.setMidLineWidth(1)
        self.sep_format.setFrameShape(QtWidgets.QFrame.VLine)
        self.sep_format.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.sep_format.setObjectName("sep_format")
        self.horizontalLayout_2.addWidget(self.sep_format)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.ch_native = QtWidgets.QCheckBox(self.group_types)
        self.ch_native.setObjectName("ch_native")
        self.verticalLayout_2.addWidget(self.ch_native)
        self.ch_posix32 = QtWidgets.QCheckBox(self.group_types)
        self.ch_posix32.setObjectName("ch_posix32")
        self.verticalLayout_2.addWidget(self.ch_posix32)
        self.ch_posix64 = QtWidgets.QCheckBox(self.group_types)
        self.ch_posix64.setObjectName("ch_posix64")
        self.verticalLayout_2.addWidget(self.ch_posix64)
        self.ch_win32 = QtWidgets.QCheckBox(self.group_types)
        self.ch_win32.setObjectName("ch_win32")
        self.verticalLayout_2.addWidget(self.ch_win32)
        self.ch_win64 = QtWidgets.QCheckBox(self.group_types)
        self.ch_win64.setObjectName("ch_win64")
        self.verticalLayout_2.addWidget(self.ch_win64)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem3)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.horizontalLayout_3.addWidget(self.group_types)
        spacerItem4 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.group_tools = QtWidgets.QGroupBox(PluginRefreshW)
        self.group_tools.setAlignment(QtCore.Qt.AlignCenter)
        self.group_tools.setFlat(True)
        self.group_tools.setObjectName("group_tools")
        self.gridLayout = QtWidgets.QGridLayout(self.group_tools)
        self.gridLayout.setObjectName("gridLayout")
        self.ico_native = QtWidgets.QLabel(self.group_tools)
        self.ico_native.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_native.setText("")
        self.ico_native.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_native.setScaledContents(True)
        self.ico_native.setObjectName("ico_native")
        self.gridLayout.addWidget(self.ico_native, 0, 0, 1, 1)
        self.ico_posix32 = QtWidgets.QLabel(self.group_tools)
        self.ico_posix32.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_posix32.setText("")
        self.ico_posix32.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_posix32.setScaledContents(True)
        self.ico_posix32.setObjectName("ico_posix32")
        self.gridLayout.addWidget(self.ico_posix32, 1, 0, 1, 1)
        self.label_rdflib = QtWidgets.QLabel(self.group_tools)
        self.label_rdflib.setObjectName("label_rdflib")
        self.gridLayout.addWidget(self.label_rdflib, 5, 1, 1, 1)
        self.label_win64 = QtWidgets.QLabel(self.group_tools)
        self.label_win64.setObjectName("label_win64")
        self.gridLayout.addWidget(self.label_win64, 4, 1, 1, 1)
        self.label_native = QtWidgets.QLabel(self.group_tools)
        self.label_native.setObjectName("label_native")
        self.gridLayout.addWidget(self.label_native, 0, 1, 1, 1)
        self.ico_posix64 = QtWidgets.QLabel(self.group_tools)
        self.ico_posix64.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_posix64.setText("")
        self.ico_posix64.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_posix64.setScaledContents(True)
        self.ico_posix64.setObjectName("ico_posix64")
        self.gridLayout.addWidget(self.ico_posix64, 2, 0, 1, 1)
        self.ico_rdflib = QtWidgets.QLabel(self.group_tools)
        self.ico_rdflib.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_rdflib.setText("")
        self.ico_rdflib.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_rdflib.setScaledContents(True)
        self.ico_rdflib.setObjectName("ico_rdflib")
        self.gridLayout.addWidget(self.ico_rdflib, 5, 0, 1, 1)
        self.label_posix32 = QtWidgets.QLabel(self.group_tools)
        self.label_posix32.setObjectName("label_posix32")
        self.gridLayout.addWidget(self.label_posix32, 1, 1, 1, 1)
        self.ico_win32 = QtWidgets.QLabel(self.group_tools)
        self.ico_win32.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_win32.setText("")
        self.ico_win32.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_win32.setScaledContents(True)
        self.ico_win32.setObjectName("ico_win32")
        self.gridLayout.addWidget(self.ico_win32, 3, 0, 1, 1)
        self.ico_win64 = QtWidgets.QLabel(self.group_tools)
        self.ico_win64.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_win64.setText("")
        self.ico_win64.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_win64.setScaledContents(True)
        self.ico_win64.setObjectName("ico_win64")
        self.gridLayout.addWidget(self.ico_win64, 4, 0, 1, 1)
        self.label_posix64 = QtWidgets.QLabel(self.group_tools)
        self.label_posix64.setObjectName("label_posix64")
        self.gridLayout.addWidget(self.label_posix64, 2, 1, 1, 1)
        self.label_win32 = QtWidgets.QLabel(self.group_tools)
        self.label_win32.setObjectName("label_win32")
        self.gridLayout.addWidget(self.label_win32, 3, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.group_tools)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem5)
        self.group_options = QtWidgets.QGroupBox(PluginRefreshW)
        self.group_options.setAlignment(QtCore.Qt.AlignCenter)
        self.group_options.setFlat(True)
        self.group_options.setObjectName("group_options")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.group_options)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.ch_do_checks = QtWidgets.QCheckBox(self.group_options)
        self.ch_do_checks.setObjectName("ch_do_checks")
        self.verticalLayout_4.addWidget(self.ch_do_checks)
        self.verticalLayout_3.addWidget(self.group_options)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem6)
        self.horizontalLayout_3.addLayout(self.verticalLayout_3)
        spacerItem7 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem7)
        self.verticalLayout_5.addLayout(self.horizontalLayout_3)
        spacerItem8 = QtWidgets.QSpacerItem(20, 6, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem8)
        self.lv2_restart_notice = QtWidgets.QWidget(PluginRefreshW)
        self.lv2_restart_notice.setObjectName("lv2_restart_notice")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.lv2_restart_notice)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(self.lv2_restart_notice)
        self.label_4.setMaximumSize(QtCore.QSize(22, 22))
        self.label_4.setText("")
        self.label_4.setPixmap(QtGui.QPixmap(":/16x16/dialog-information.svgz"))
        self.label_4.setScaledContents(True)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.label_5 = QtWidgets.QLabel(self.lv2_restart_notice)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_4.addWidget(self.label_5)
        self.verticalLayout_5.addWidget(self.lv2_restart_notice)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.progressBar = QtWidgets.QProgressBar(PluginRefreshW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.progressBar.sizePolicy().hasHeightForWidth())
        self.progressBar.setSizePolicy(sizePolicy)
        self.progressBar.setMaximum(100)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setObjectName("progressBar")
        self.horizontalLayout.addWidget(self.progressBar)
        self.b_start = QtWidgets.QPushButton(PluginRefreshW)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/arrow-right.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_start.setIcon(icon)
        self.b_start.setObjectName("b_start")
        self.horizontalLayout.addWidget(self.b_start)
        self.b_skip = QtWidgets.QPushButton(PluginRefreshW)
        self.b_skip.setObjectName("b_skip")
        self.horizontalLayout.addWidget(self.b_skip)
        self.b_close = QtWidgets.QPushButton(PluginRefreshW)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/16x16/window-close.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_close.setIcon(icon1)
        self.b_close.setObjectName("b_close")
        self.horizontalLayout.addWidget(self.b_close)
        self.verticalLayout_5.addLayout(self.horizontalLayout)

        self.retranslateUi(PluginRefreshW)
        self.b_close.clicked.connect(PluginRefreshW.close) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(PluginRefreshW)

    def retranslateUi(self, PluginRefreshW):
        _translate = QtCore.QCoreApplication.translate
        PluginRefreshW.setWindowTitle(_translate("PluginRefreshW", "Carla - Refresh"))
        self.group_types.setTitle(_translate("PluginRefreshW", "Search for new..."))
        self.ch_ladspa.setText(_translate("PluginRefreshW", "LADSPA"))
        self.ch_dssi.setText(_translate("PluginRefreshW", "DSSI"))
        self.ch_lv2.setText(_translate("PluginRefreshW", "LV2"))
        self.ch_vst.setText(_translate("PluginRefreshW", "VST2"))
        self.ch_vst3.setText(_translate("PluginRefreshW", "VST3"))
        self.ch_au.setText(_translate("PluginRefreshW", "AU"))
        self.ch_jsfx.setText(_translate("PluginRefreshW", "JSFX"))
        self.ch_sf2.setText(_translate("PluginRefreshW", "SF2/3"))
        self.ch_sfz.setText(_translate("PluginRefreshW", "SFZ"))
        self.ch_native.setText(_translate("PluginRefreshW", "Native"))
        self.ch_posix32.setText(_translate("PluginRefreshW", "POSIX 32bit"))
        self.ch_posix64.setText(_translate("PluginRefreshW", "POSIX 64bit"))
        self.ch_win32.setText(_translate("PluginRefreshW", "Windows 32bit"))
        self.ch_win64.setText(_translate("PluginRefreshW", "Windows 64bit"))
        self.group_tools.setTitle(_translate("PluginRefreshW", "Available tools:"))
        self.label_rdflib.setText(_translate("PluginRefreshW", "python3-rdflib (LADSPA-RDF support)"))
        self.label_win64.setText(_translate("PluginRefreshW", "carla-discovery-win64"))
        self.label_native.setText(_translate("PluginRefreshW", "carla-discovery-native"))
        self.label_posix32.setText(_translate("PluginRefreshW", "carla-discovery-posix32"))
        self.label_posix64.setText(_translate("PluginRefreshW", "carla-discovery-posix64"))
        self.label_win32.setText(_translate("PluginRefreshW", "carla-discovery-win32"))
        self.group_options.setTitle(_translate("PluginRefreshW", "Options:"))
        self.ch_do_checks.setToolTip(_translate("PluginRefreshW", "Carla will run small processing checks when scanning the plugins (to make sure they won\'t crash).\n"
"You can disable these checks to get a faster scanning time (at your own risk)."))
        self.ch_do_checks.setText(_translate("PluginRefreshW", "Run processing checks while scanning"))
        self.label_5.setText(_translate("PluginRefreshW", "Restart Carla to list the new LV2 plugins"))
        self.progressBar.setFormat(_translate("PluginRefreshW", "Press \'Scan\' to begin the search"))
        self.b_start.setText(_translate("PluginRefreshW", "Scan"))
        self.b_skip.setText(_translate("PluginRefreshW", ">> Skip"))
        self.b_close.setText(_translate("PluginRefreshW", "Close"))
import resources_rc
