# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_plugin_calf.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginWidget(object):
    def setupUi(self, PluginWidget):
        PluginWidget.setObjectName("PluginWidget")
        PluginWidget.resize(622, 90)
        PluginWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.verticalLayout = QtWidgets.QVBoxLayout(PluginWidget)
        self.verticalLayout.setContentsMargins(4, 3, 6, 3)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.layout_top = QtWidgets.QHBoxLayout()
        self.layout_top.setSpacing(1)
        self.layout_top.setObjectName("layout_top")
        self.w_buttons = QtWidgets.QWidget(PluginWidget)
        self.w_buttons.setMinimumSize(QtCore.QSize(80, 0))
        self.w_buttons.setMaximumSize(QtCore.QSize(80, 16777215))
        self.w_buttons.setObjectName("w_buttons")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.w_buttons)
        self.horizontalLayout_5.setContentsMargins(12, 0, 0, 0)
        self.horizontalLayout_5.setSpacing(2)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.b_enable = ScalableButton(self.w_buttons)
        self.b_enable.setCheckable(True)
        self.b_enable.setObjectName("b_enable")
        self.horizontalLayout_5.addWidget(self.b_enable)
        self.label_active = QtWidgets.QLabel(self.w_buttons)
        self.label_active.setObjectName("label_active")
        self.horizontalLayout_5.addWidget(self.label_active)
        self.layout_top.addWidget(self.w_buttons)
        self.line_sep = QtWidgets.QFrame(PluginWidget)
        self.line_sep.setMinimumSize(QtCore.QSize(0, 20))
        self.line_sep.setMaximumSize(QtCore.QSize(16777215, 20))
        self.line_sep.setLineWidth(0)
        self.line_sep.setMidLineWidth(1)
        self.line_sep.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_sep.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_sep.setObjectName("line_sep")
        self.layout_top.addWidget(self.line_sep)
        spacerItem = QtWidgets.QSpacerItem(8, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.layout_top.addItem(spacerItem)
        self.label_name = QtWidgets.QLabel(PluginWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_name.setFont(font)
        self.label_name.setAlignment(QtCore.Qt.AlignCenter)
        self.label_name.setObjectName("label_name")
        self.layout_top.addWidget(self.label_name)
        spacerItem1 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_top.addItem(spacerItem1)
        self.layout_midi = QtWidgets.QVBoxLayout()
        self.layout_midi.setContentsMargins(-1, -1, 4, -1)
        self.layout_midi.setSpacing(0)
        self.layout_midi.setObjectName("layout_midi")
        self.label_midi = QtWidgets.QLabel(PluginWidget)
        self.label_midi.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignHCenter)
        self.label_midi.setObjectName("label_midi")
        self.layout_midi.addWidget(self.label_midi)
        self.led_midi = LEDButton(PluginWidget)
        self.led_midi.setMinimumSize(QtCore.QSize(25, 25))
        self.led_midi.setMaximumSize(QtCore.QSize(25, 25))
        self.led_midi.setText("")
        self.led_midi.setIconSize(QtCore.QSize(25, 25))
        self.led_midi.setCheckable(True)
        self.led_midi.setObjectName("led_midi")
        self.layout_midi.addWidget(self.led_midi)
        self.layout_top.addLayout(self.layout_midi)
        self.layout_peak_in = QtWidgets.QVBoxLayout()
        self.layout_peak_in.setSpacing(0)
        self.layout_peak_in.setObjectName("layout_peak_in")
        self.label_audio_in = QtWidgets.QLabel(PluginWidget)
        self.label_audio_in.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignHCenter)
        self.label_audio_in.setObjectName("label_audio_in")
        self.layout_peak_in.addWidget(self.label_audio_in)
        self.peak_in = DigitalPeakMeter(PluginWidget)
        self.peak_in.setMinimumSize(QtCore.QSize(150, 0))
        self.peak_in.setObjectName("peak_in")
        self.layout_peak_in.addWidget(self.peak_in)
        self.layout_top.addLayout(self.layout_peak_in)
        self.layout_peak_out = QtWidgets.QVBoxLayout()
        self.layout_peak_out.setSpacing(0)
        self.layout_peak_out.setObjectName("layout_peak_out")
        self.label_audio_out = QtWidgets.QLabel(PluginWidget)
        self.label_audio_out.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignHCenter)
        self.label_audio_out.setObjectName("label_audio_out")
        self.layout_peak_out.addWidget(self.label_audio_out)
        self.peak_out = DigitalPeakMeter(PluginWidget)
        self.peak_out.setMinimumSize(QtCore.QSize(150, 0))
        self.peak_out.setObjectName("peak_out")
        self.layout_peak_out.addWidget(self.peak_out)
        self.layout_top.addLayout(self.layout_peak_out)
        self.verticalLayout.addLayout(self.layout_top)
        self.line = QtWidgets.QFrame(PluginWidget)
        self.line.setLineWidth(0)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.layout_bottom = QtWidgets.QHBoxLayout()
        self.layout_bottom.setContentsMargins(6, 4, 4, -1)
        self.layout_bottom.setSpacing(1)
        self.layout_bottom.setObjectName("layout_bottom")
        self.w_buttons2 = QtWidgets.QHBoxLayout()
        self.w_buttons2.setSpacing(3)
        self.w_buttons2.setObjectName("w_buttons2")
        self.b_gui = ScalableButton(PluginWidget)
        self.b_gui.setCheckable(True)
        self.b_gui.setObjectName("b_gui")
        self.w_buttons2.addWidget(self.b_gui)
        self.b_edit = ScalableButton(PluginWidget)
        self.b_edit.setCheckable(True)
        self.b_edit.setObjectName("b_edit")
        self.w_buttons2.addWidget(self.b_edit)
        self.b_remove = ScalableButton(PluginWidget)
        self.b_remove.setObjectName("b_remove")
        self.w_buttons2.addWidget(self.b_remove)
        self.layout_bottom.addLayout(self.w_buttons2)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_bottom.addItem(spacerItem2)
        self.w_knobs = QtWidgets.QWidget(PluginWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.w_knobs.sizePolicy().hasHeightForWidth())
        self.w_knobs.setSizePolicy(sizePolicy)
        self.w_knobs.setObjectName("w_knobs")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.w_knobs)
        self.horizontalLayout_4.setContentsMargins(4, 0, 4, 0)
        self.horizontalLayout_4.setSpacing(2)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.layout_bottom.addWidget(self.w_knobs)
        self.label_logo = QtWidgets.QLabel(PluginWidget)
        self.label_logo.setMinimumSize(QtCore.QSize(71, 30))
        self.label_logo.setMaximumSize(QtCore.QSize(71, 30))
        self.label_logo.setText("")
        self.label_logo.setPixmap(QtGui.QPixmap(":/bitmaps/logo_calf.png"))
        self.label_logo.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_logo.setObjectName("label_logo")
        self.layout_bottom.addWidget(self.label_logo)
        self.verticalLayout.addLayout(self.layout_bottom)

        self.retranslateUi(PluginWidget)
        QtCore.QMetaObject.connectSlotsByName(PluginWidget)

    def retranslateUi(self, PluginWidget):
        _translate = QtCore.QCoreApplication.translate
        PluginWidget.setWindowTitle(_translate("PluginWidget", "Frame"))
        self.b_enable.setText(_translate("PluginWidget", "Enable"))
        self.label_active.setText(_translate("PluginWidget", "On/Off"))
        self.label_name.setText(_translate("PluginWidget", "PluginName"))
        self.label_midi.setText(_translate("PluginWidget", "MIDI"))
        self.label_audio_in.setText(_translate("PluginWidget", "AUDIO IN"))
        self.label_audio_out.setText(_translate("PluginWidget", "AUDIO OUT"))
        self.b_gui.setText(_translate("PluginWidget", "GUI"))
        self.b_edit.setText(_translate("PluginWidget", "Edit"))
        self.b_remove.setText(_translate("PluginWidget", "Remove"))
from widgets.digitalpeakmeter import DigitalPeakMeter
from widgets.ledbutton import LEDButton
from widgets.scalablebutton import ScalableButton
import resources_rc
