# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_database.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginDatabaseW(object):
    def setupUi(self, PluginDatabaseW):
        PluginDatabaseW.setObjectName("PluginDatabaseW")
        PluginDatabaseW.resize(1100, 683)
        self.gridLayout_3 = QtWidgets.QGridLayout(PluginDatabaseW)
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 2, 0, 1, 1)
        self.tab_reqs = QtWidgets.QTabWidget(PluginDatabaseW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tab_reqs.sizePolicy().hasHeightForWidth())
        self.tab_reqs.setSizePolicy(sizePolicy)
        self.tab_reqs.setProperty("tabBarAutoHide", True)
        self.tab_reqs.setObjectName("tab_reqs")
        self.tw_reqs = QtWidgets.QWidget()
        self.tw_reqs.setObjectName("tw_reqs")
        self.gridLayout = QtWidgets.QGridLayout(self.tw_reqs)
        self.gridLayout.setObjectName("gridLayout")
        self.l_reqs = QtWidgets.QLabel(self.tw_reqs)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.l_reqs.setFont(font)
        self.l_reqs.setObjectName("l_reqs")
        self.gridLayout.addWidget(self.l_reqs, 0, 1, 1, 1)
        self.ch_gui = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_gui.setObjectName("ch_gui")
        self.gridLayout.addWidget(self.ch_gui, 5, 1, 1, 1)
        self.ch_cv = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_cv.setObjectName("ch_cv")
        self.gridLayout.addWidget(self.ch_cv, 4, 1, 1, 1)
        self.ch_rtsafe = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_rtsafe.setObjectName("ch_rtsafe")
        self.gridLayout.addWidget(self.ch_rtsafe, 2, 1, 1, 1)
        self.ch_stereo = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_stereo.setObjectName("ch_stereo")
        self.gridLayout.addWidget(self.ch_stereo, 3, 1, 1, 1)
        self.ch_inline_display = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_inline_display.setObjectName("ch_inline_display")
        self.gridLayout.addWidget(self.ch_inline_display, 6, 1, 1, 1)
        self.ch_favorites = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_favorites.setObjectName("ch_favorites")
        self.gridLayout.addWidget(self.ch_favorites, 1, 1, 1, 1)
        self.tab_reqs.addTab(self.tw_reqs, "")
        self.gridLayout_3.addWidget(self.tab_reqs, 3, 0, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(PluginDatabaseW)
        self.label.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.b_add = QtWidgets.QPushButton(PluginDatabaseW)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/list-add.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_add.setIcon(icon)
        self.b_add.setObjectName("b_add")
        self.horizontalLayout_2.addWidget(self.b_add)
        self.b_cancel = QtWidgets.QPushButton(PluginDatabaseW)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/16x16/dialog-cancel.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_cancel.setIcon(icon1)
        self.b_cancel.setObjectName("b_cancel")
        self.horizontalLayout_2.addWidget(self.b_cancel)
        self.gridLayout_3.addLayout(self.horizontalLayout_2, 4, 0, 1, 3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEdit = QtWidgets.QLineEdit(PluginDatabaseW)
        self.lineEdit.setProperty("clearButtonEnabled", True)
        self.lineEdit.setObjectName("lineEdit")
        self.horizontalLayout.addWidget(self.lineEdit)
        self.b_refresh = QtWidgets.QPushButton(PluginDatabaseW)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/16x16/view-refresh.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_refresh.setIcon(icon2)
        self.b_refresh.setObjectName("b_refresh")
        self.horizontalLayout.addWidget(self.b_refresh)
        self.b_clear_filters = QtWidgets.QPushButton(PluginDatabaseW)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/16x16/edit-clear.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_clear_filters.setIcon(icon3)
        self.b_clear_filters.setObjectName("b_clear_filters")
        self.horizontalLayout.addWidget(self.b_clear_filters)
        self.gridLayout_3.addLayout(self.horizontalLayout, 0, 0, 1, 3)
        self.tab_info = QtWidgets.QTabWidget(PluginDatabaseW)
        self.tab_info.setProperty("tabBarAutoHide", True)
        self.tab_info.setObjectName("tab_info")
        self.tw_info = QtWidgets.QWidget()
        self.tw_info.setObjectName("tw_info")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tw_info)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem2, 19, 0, 1, 1)
        self.l_format = QtWidgets.QLabel(self.tw_info)
        self.l_format.setObjectName("l_format")
        self.gridLayout_2.addWidget(self.l_format, 1, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.tw_info)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 1, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.tw_info)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 3, 0, 1, 1)
        self.l_arch = QtWidgets.QLabel(self.tw_info)
        self.l_arch.setObjectName("l_arch")
        self.gridLayout_2.addWidget(self.l_arch, 3, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.tw_info)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 2, 0, 1, 1)
        self.l_type = QtWidgets.QLabel(self.tw_info)
        self.l_type.setObjectName("l_type")
        self.gridLayout_2.addWidget(self.l_type, 2, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem3, 19, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.tw_info)
        self.label_12.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_12.setObjectName("label_12")
        self.gridLayout_2.addWidget(self.label_12, 10, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.tw_info)
        self.label_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_8.setObjectName("label_8")
        self.gridLayout_2.addWidget(self.label_8, 6, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.tw_info)
        self.label_11.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 9, 0, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.tw_info)
        self.label_13.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_13.setObjectName("label_13")
        self.gridLayout_2.addWidget(self.label_13, 11, 0, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.tw_info)
        self.label_14.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_14.setObjectName("label_14")
        self.gridLayout_2.addWidget(self.label_14, 12, 0, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.tw_info)
        self.label_15.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_15.setObjectName("label_15")
        self.gridLayout_2.addWidget(self.label_15, 13, 0, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.tw_info)
        self.label_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_9.setObjectName("label_9")
        self.gridLayout_2.addWidget(self.label_9, 7, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.tw_info)
        self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 8, 0, 1, 1)
        self.la_id = QtWidgets.QLabel(self.tw_info)
        self.la_id.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.la_id.setObjectName("la_id")
        self.gridLayout_2.addWidget(self.la_id, 4, 0, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.tw_info)
        self.label_18.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_18.setObjectName("label_18")
        self.gridLayout_2.addWidget(self.label_18, 16, 0, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.tw_info)
        self.label_17.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_17.setObjectName("label_17")
        self.gridLayout_2.addWidget(self.label_17, 15, 0, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.tw_info)
        self.label_19.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_19.setObjectName("label_19")
        self.gridLayout_2.addWidget(self.label_19, 18, 0, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.tw_info)
        self.label_20.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_20.setObjectName("label_20")
        self.gridLayout_2.addWidget(self.label_20, 17, 0, 1, 1)
        self.l_aouts = QtWidgets.QLabel(self.tw_info)
        self.l_aouts.setObjectName("l_aouts")
        self.gridLayout_2.addWidget(self.l_aouts, 7, 1, 1, 1)
        self.l_id = QtWidgets.QLabel(self.tw_info)
        self.l_id.setObjectName("l_id")
        self.gridLayout_2.addWidget(self.l_id, 4, 1, 1, 1)
        self.l_ains = QtWidgets.QLabel(self.tw_info)
        self.l_ains.setObjectName("l_ains")
        self.gridLayout_2.addWidget(self.l_ains, 6, 1, 1, 1)
        self.l_synth = QtWidgets.QLabel(self.tw_info)
        self.l_synth.setObjectName("l_synth")
        self.gridLayout_2.addWidget(self.l_synth, 18, 1, 1, 1)
        self.l_pins = QtWidgets.QLabel(self.tw_info)
        self.l_pins.setObjectName("l_pins")
        self.gridLayout_2.addWidget(self.l_pins, 12, 1, 1, 1)
        self.l_cvins = QtWidgets.QLabel(self.tw_info)
        self.l_cvins.setObjectName("l_cvins")
        self.gridLayout_2.addWidget(self.l_cvins, 8, 1, 1, 1)
        self.l_pouts = QtWidgets.QLabel(self.tw_info)
        self.l_pouts.setObjectName("l_pouts")
        self.gridLayout_2.addWidget(self.l_pouts, 13, 1, 1, 1)
        self.l_mouts = QtWidgets.QLabel(self.tw_info)
        self.l_mouts.setObjectName("l_mouts")
        self.gridLayout_2.addWidget(self.l_mouts, 11, 1, 1, 1)
        self.l_mins = QtWidgets.QLabel(self.tw_info)
        self.l_mins.setObjectName("l_mins")
        self.gridLayout_2.addWidget(self.l_mins, 10, 1, 1, 1)
        self.l_bridged = QtWidgets.QLabel(self.tw_info)
        self.l_bridged.setObjectName("l_bridged")
        self.gridLayout_2.addWidget(self.l_bridged, 17, 1, 1, 1)
        self.l_idisp = QtWidgets.QLabel(self.tw_info)
        self.l_idisp.setObjectName("l_idisp")
        self.gridLayout_2.addWidget(self.l_idisp, 16, 1, 1, 1)
        self.l_gui = QtWidgets.QLabel(self.tw_info)
        self.l_gui.setObjectName("l_gui")
        self.gridLayout_2.addWidget(self.l_gui, 15, 1, 1, 1)
        self.l_cvouts = QtWidgets.QLabel(self.tw_info)
        self.l_cvouts.setObjectName("l_cvouts")
        self.gridLayout_2.addWidget(self.l_cvouts, 9, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.tw_info)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 0, 0, 1, 2)
        self.line_2 = QtWidgets.QFrame(self.tw_info)
        self.line_2.setLineWidth(0)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout_2.addWidget(self.line_2, 14, 0, 1, 2)
        self.line = QtWidgets.QFrame(self.tw_info)
        self.line.setLineWidth(0)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_2.addWidget(self.line, 5, 0, 1, 2)
        self.tab_info.addTab(self.tw_info, "")
        self.gridLayout_3.addWidget(self.tab_info, 1, 2, 3, 1)
        self.tableWidget = QtWidgets.QTableWidget(PluginDatabaseW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tableWidget.sizePolicy().hasHeightForWidth())
        self.tableWidget.setSizePolicy(sizePolicy)
        self.tableWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidget.setProperty("showDropIndicator", False)
        self.tableWidget.setDragDropOverwriteMode(False)
        self.tableWidget.setAlternatingRowColors(True)
        self.tableWidget.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tableWidget.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableWidget.setShowGrid(False)
        self.tableWidget.setGridStyle(QtCore.Qt.NoPen)
        self.tableWidget.setWordWrap(False)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(5)
        self.tableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/16x16/bookmarks.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon4)
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(4, item)
        self.tableWidget.horizontalHeader().setMinimumSectionSize(24)
        self.tableWidget.horizontalHeader().setStretchLastSection(True)
        self.tableWidget.verticalHeader().setVisible(False)
        self.tableWidget.verticalHeader().setDefaultSectionSize(22)
        self.tableWidget.verticalHeader().setMinimumSectionSize(12)
        self.gridLayout_3.addWidget(self.tableWidget, 1, 1, 3, 1)
        self.toolBox = QtWidgets.QToolBox(PluginDatabaseW)
        self.toolBox.setObjectName("toolBox")
        self.p_format = QtWidgets.QWidget()
        self.p_format.setGeometry(QtCore.QRect(0, 0, 158, 237))
        self.p_format.setObjectName("p_format")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.p_format)
        self.verticalLayout.setObjectName("verticalLayout")
        self.ch_internal = QtWidgets.QCheckBox(self.p_format)
        self.ch_internal.setObjectName("ch_internal")
        self.verticalLayout.addWidget(self.ch_internal)
        self.ch_ladspa = QtWidgets.QCheckBox(self.p_format)
        self.ch_ladspa.setObjectName("ch_ladspa")
        self.verticalLayout.addWidget(self.ch_ladspa)
        self.ch_dssi = QtWidgets.QCheckBox(self.p_format)
        self.ch_dssi.setObjectName("ch_dssi")
        self.verticalLayout.addWidget(self.ch_dssi)
        self.ch_lv2 = QtWidgets.QCheckBox(self.p_format)
        self.ch_lv2.setObjectName("ch_lv2")
        self.verticalLayout.addWidget(self.ch_lv2)
        self.ch_vst = QtWidgets.QCheckBox(self.p_format)
        self.ch_vst.setObjectName("ch_vst")
        self.verticalLayout.addWidget(self.ch_vst)
        self.ch_vst3 = QtWidgets.QCheckBox(self.p_format)
        self.ch_vst3.setObjectName("ch_vst3")
        self.verticalLayout.addWidget(self.ch_vst3)
        self.ch_au = QtWidgets.QCheckBox(self.p_format)
        self.ch_au.setObjectName("ch_au")
        self.verticalLayout.addWidget(self.ch_au)
        self.ch_jsfx = QtWidgets.QCheckBox(self.p_format)
        self.ch_jsfx.setObjectName("ch_jsfx")
        self.verticalLayout.addWidget(self.ch_jsfx)
        self.ch_kits = QtWidgets.QCheckBox(self.p_format)
        self.ch_kits.setObjectName("ch_kits")
        self.verticalLayout.addWidget(self.ch_kits)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem4)
        self.toolBox.addItem(self.p_format, "")
        self.p_type = QtWidgets.QWidget()
        self.p_type.setGeometry(QtCore.QRect(0, 0, 158, 156))
        self.p_type.setObjectName("p_type")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.p_type)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.ch_effects = QtWidgets.QCheckBox(self.p_type)
        self.ch_effects.setObjectName("ch_effects")
        self.verticalLayout_2.addWidget(self.ch_effects)
        self.ch_instruments = QtWidgets.QCheckBox(self.p_type)
        self.ch_instruments.setObjectName("ch_instruments")
        self.verticalLayout_2.addWidget(self.ch_instruments)
        self.ch_midi = QtWidgets.QCheckBox(self.p_type)
        self.ch_midi.setObjectName("ch_midi")
        self.verticalLayout_2.addWidget(self.ch_midi)
        self.ch_other = QtWidgets.QCheckBox(self.p_type)
        self.ch_other.setObjectName("ch_other")
        self.verticalLayout_2.addWidget(self.ch_other)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem5)
        self.toolBox.addItem(self.p_type, "")
        self.p_category = QtWidgets.QWidget()
        self.p_category.setObjectName("p_category")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.p_category)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.ch_cat_all = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_all.setObjectName("ch_cat_all")
        self.verticalLayout_4.addWidget(self.ch_cat_all)
        self.ch_cat_delay = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_delay.setObjectName("ch_cat_delay")
        self.verticalLayout_4.addWidget(self.ch_cat_delay)
        self.ch_cat_distortion = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_distortion.setObjectName("ch_cat_distortion")
        self.verticalLayout_4.addWidget(self.ch_cat_distortion)
        self.ch_cat_dynamics = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_dynamics.setObjectName("ch_cat_dynamics")
        self.verticalLayout_4.addWidget(self.ch_cat_dynamics)
        self.ch_cat_eq = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_eq.setObjectName("ch_cat_eq")
        self.verticalLayout_4.addWidget(self.ch_cat_eq)
        self.ch_cat_filter = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_filter.setObjectName("ch_cat_filter")
        self.verticalLayout_4.addWidget(self.ch_cat_filter)
        self.ch_cat_modulator = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_modulator.setObjectName("ch_cat_modulator")
        self.verticalLayout_4.addWidget(self.ch_cat_modulator)
        self.ch_cat_synth = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_synth.setObjectName("ch_cat_synth")
        self.verticalLayout_4.addWidget(self.ch_cat_synth)
        self.ch_cat_utility = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_utility.setObjectName("ch_cat_utility")
        self.verticalLayout_4.addWidget(self.ch_cat_utility)
        self.ch_cat_other = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_other.setObjectName("ch_cat_other")
        self.verticalLayout_4.addWidget(self.ch_cat_other)
        spacerItem6 = QtWidgets.QSpacerItem(20, 23, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem6)
        self.toolBox.addItem(self.p_category, "")
        self.p_arch = QtWidgets.QWidget()
        self.p_arch.setGeometry(QtCore.QRect(0, 0, 158, 130))
        self.p_arch.setObjectName("p_arch")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.p_arch)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.ch_native = QtWidgets.QCheckBox(self.p_arch)
        self.ch_native.setObjectName("ch_native")
        self.verticalLayout_3.addWidget(self.ch_native)
        self.ch_bridged = QtWidgets.QCheckBox(self.p_arch)
        self.ch_bridged.setObjectName("ch_bridged")
        self.verticalLayout_3.addWidget(self.ch_bridged)
        self.ch_bridged_wine = QtWidgets.QCheckBox(self.p_arch)
        self.ch_bridged_wine.setObjectName("ch_bridged_wine")
        self.verticalLayout_3.addWidget(self.ch_bridged_wine)
        spacerItem7 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem7)
        self.toolBox.addItem(self.p_arch, "")
        self.gridLayout_3.addWidget(self.toolBox, 1, 0, 1, 1)
        self.act_focus_search = QtWidgets.QAction(PluginDatabaseW)
        self.act_focus_search.setObjectName("act_focus_search")

        self.retranslateUi(PluginDatabaseW)
        self.toolBox.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(PluginDatabaseW)
        PluginDatabaseW.setTabOrder(self.lineEdit, self.tableWidget)
        PluginDatabaseW.setTabOrder(self.tableWidget, self.b_add)
        PluginDatabaseW.setTabOrder(self.b_add, self.b_cancel)
        PluginDatabaseW.setTabOrder(self.b_cancel, self.b_refresh)
        PluginDatabaseW.setTabOrder(self.b_refresh, self.b_clear_filters)
        PluginDatabaseW.setTabOrder(self.b_clear_filters, self.ch_internal)
        PluginDatabaseW.setTabOrder(self.ch_internal, self.ch_ladspa)
        PluginDatabaseW.setTabOrder(self.ch_ladspa, self.ch_dssi)
        PluginDatabaseW.setTabOrder(self.ch_dssi, self.ch_lv2)
        PluginDatabaseW.setTabOrder(self.ch_lv2, self.ch_vst)
        PluginDatabaseW.setTabOrder(self.ch_vst, self.ch_vst3)
        PluginDatabaseW.setTabOrder(self.ch_vst3, self.ch_au)
        PluginDatabaseW.setTabOrder(self.ch_au, self.ch_kits)
        PluginDatabaseW.setTabOrder(self.ch_kits, self.ch_effects)
        PluginDatabaseW.setTabOrder(self.ch_effects, self.ch_instruments)
        PluginDatabaseW.setTabOrder(self.ch_instruments, self.ch_midi)
        PluginDatabaseW.setTabOrder(self.ch_midi, self.ch_other)
        PluginDatabaseW.setTabOrder(self.ch_other, self.ch_native)
        PluginDatabaseW.setTabOrder(self.ch_native, self.ch_bridged)
        PluginDatabaseW.setTabOrder(self.ch_bridged, self.ch_bridged_wine)
        PluginDatabaseW.setTabOrder(self.ch_bridged_wine, self.ch_inline_display)
        PluginDatabaseW.setTabOrder(self.ch_inline_display, self.ch_stereo)
        PluginDatabaseW.setTabOrder(self.ch_stereo, self.ch_rtsafe)
        PluginDatabaseW.setTabOrder(self.ch_rtsafe, self.ch_gui)
        PluginDatabaseW.setTabOrder(self.ch_gui, self.ch_cv)
        PluginDatabaseW.setTabOrder(self.ch_cv, self.tab_reqs)
        PluginDatabaseW.setTabOrder(self.tab_reqs, self.tab_info)

    def retranslateUi(self, PluginDatabaseW):
        _translate = QtCore.QCoreApplication.translate
        PluginDatabaseW.setWindowTitle(_translate("PluginDatabaseW", "Carla - Add New"))
        self.l_reqs.setText(_translate("PluginDatabaseW", "Requirements"))
        self.ch_gui.setText(_translate("PluginDatabaseW", "With Custom GUI "))
        self.ch_cv.setText(_translate("PluginDatabaseW", "With CV Ports"))
        self.ch_rtsafe.setText(_translate("PluginDatabaseW", "Real-time safe only"))
        self.ch_stereo.setText(_translate("PluginDatabaseW", "Stereo only"))
        self.ch_inline_display.setText(_translate("PluginDatabaseW", "With Inline Display"))
        self.ch_favorites.setText(_translate("PluginDatabaseW", "Favorites only"))
        self.label.setText(_translate("PluginDatabaseW", "(Number of Plugins go here)"))
        self.b_add.setText(_translate("PluginDatabaseW", "&Add Plugin"))
        self.b_cancel.setText(_translate("PluginDatabaseW", "Cancel"))
        self.b_refresh.setText(_translate("PluginDatabaseW", "Refresh"))
        self.b_clear_filters.setText(_translate("PluginDatabaseW", "Reset filters"))
        self.l_format.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.label_2.setText(_translate("PluginDatabaseW", "Format:"))
        self.label_4.setText(_translate("PluginDatabaseW", "Architecture:"))
        self.l_arch.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.label_6.setText(_translate("PluginDatabaseW", "Type:"))
        self.l_type.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.label_12.setText(_translate("PluginDatabaseW", "MIDI Ins:"))
        self.label_8.setText(_translate("PluginDatabaseW", "Audio Ins:"))
        self.label_11.setText(_translate("PluginDatabaseW", "CV Outs:"))
        self.label_13.setText(_translate("PluginDatabaseW", "MIDI Outs:"))
        self.label_14.setText(_translate("PluginDatabaseW", "Parameter Ins:"))
        self.label_15.setText(_translate("PluginDatabaseW", "Parameter Outs:"))
        self.label_9.setText(_translate("PluginDatabaseW", "Audio Outs:"))
        self.label_10.setText(_translate("PluginDatabaseW", "CV Ins:"))
        self.la_id.setText(_translate("PluginDatabaseW", "UniqueID:"))
        self.label_18.setText(_translate("PluginDatabaseW", "Has Inline Display:"))
        self.label_17.setText(_translate("PluginDatabaseW", "Has Custom GUI:"))
        self.label_19.setText(_translate("PluginDatabaseW", "Is Synth:"))
        self.label_20.setText(_translate("PluginDatabaseW", "Is Bridged:"))
        self.l_aouts.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_id.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_ains.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_synth.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_pins.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_cvins.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_pouts.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_mouts.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_mins.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_bridged.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_idisp.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_gui.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_cvouts.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.label_3.setText(_translate("PluginDatabaseW", "Information"))
        self.tableWidget.setSortingEnabled(True)
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(_translate("PluginDatabaseW", "Name"))
        item = self.tableWidget.horizontalHeaderItem(2)
        item.setText(_translate("PluginDatabaseW", "Label/URI"))
        item = self.tableWidget.horizontalHeaderItem(3)
        item.setText(_translate("PluginDatabaseW", "Maker"))
        item = self.tableWidget.horizontalHeaderItem(4)
        item.setText(_translate("PluginDatabaseW", "Binary/Filename"))
        self.ch_internal.setText(_translate("PluginDatabaseW", "Internal"))
        self.ch_ladspa.setText(_translate("PluginDatabaseW", "LADSPA"))
        self.ch_dssi.setText(_translate("PluginDatabaseW", "DSSI"))
        self.ch_lv2.setText(_translate("PluginDatabaseW", "LV2"))
        self.ch_vst.setText(_translate("PluginDatabaseW", "VST2"))
        self.ch_vst3.setText(_translate("PluginDatabaseW", "VST3"))
        self.ch_au.setText(_translate("PluginDatabaseW", "AU"))
        self.ch_jsfx.setText(_translate("PluginDatabaseW", "JSFX"))
        self.ch_kits.setText(_translate("PluginDatabaseW", "Sound Kits"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.p_format), _translate("PluginDatabaseW", "Format"))
        self.ch_effects.setText(_translate("PluginDatabaseW", "Effects"))
        self.ch_instruments.setText(_translate("PluginDatabaseW", "Instruments"))
        self.ch_midi.setText(_translate("PluginDatabaseW", "MIDI Plugins"))
        self.ch_other.setText(_translate("PluginDatabaseW", "Other/Misc"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.p_type), _translate("PluginDatabaseW", "Type"))
        self.ch_cat_all.setText(_translate("PluginDatabaseW", "All"))
        self.ch_cat_delay.setText(_translate("PluginDatabaseW", "Delay"))
        self.ch_cat_distortion.setText(_translate("PluginDatabaseW", "Distortion"))
        self.ch_cat_dynamics.setText(_translate("PluginDatabaseW", "Dynamics"))
        self.ch_cat_eq.setText(_translate("PluginDatabaseW", "EQ"))
        self.ch_cat_filter.setText(_translate("PluginDatabaseW", "Filter"))
        self.ch_cat_modulator.setText(_translate("PluginDatabaseW", "Modulator"))
        self.ch_cat_synth.setText(_translate("PluginDatabaseW", "Synth"))
        self.ch_cat_utility.setText(_translate("PluginDatabaseW", "Utility"))
        self.ch_cat_other.setText(_translate("PluginDatabaseW", "Other"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.p_category), _translate("PluginDatabaseW", "Category"))
        self.ch_native.setText(_translate("PluginDatabaseW", "Native"))
        self.ch_bridged.setText(_translate("PluginDatabaseW", "Bridged"))
        self.ch_bridged_wine.setText(_translate("PluginDatabaseW", "Bridged (Wine)"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.p_arch), _translate("PluginDatabaseW", "Architecture"))
        self.act_focus_search.setText(_translate("PluginDatabaseW", "Focus Text Search"))
        self.act_focus_search.setShortcut(_translate("PluginDatabaseW", "Ctrl+F"))
import resources_rc
