# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_edit.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginEdit(object):
    def setupUi(self, PluginEdit):
        PluginEdit.setObjectName("PluginEdit")
        PluginEdit.resize(537, 525)
        self.verticalLayout = QtWidgets.QVBoxLayout(PluginEdit)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(PluginEdit)
        self.tabWidget.setTabPosition(QtWidgets.QTabWidget.South)
        self.tabWidget.setObjectName("tabWidget")
        self.tabEdit = QtWidgets.QWidget()
        self.tabEdit.setObjectName("tabEdit")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tabEdit)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBox_3 = QtWidgets.QGroupBox(self.tabEdit)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.label_ctrl_channel = QtWidgets.QLabel(self.groupBox_3)
        self.label_ctrl_channel.setObjectName("label_ctrl_channel")
        self.horizontalLayout_5.addWidget(self.label_ctrl_channel)
        self.sb_ctrl_channel = QtWidgets.QSpinBox(self.groupBox_3)
        self.sb_ctrl_channel.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.sb_ctrl_channel.setAlignment(QtCore.Qt.AlignCenter)
        self.sb_ctrl_channel.setMinimum(0)
        self.sb_ctrl_channel.setMaximum(16)
        self.sb_ctrl_channel.setProperty("value", 0)
        self.sb_ctrl_channel.setObjectName("sb_ctrl_channel")
        self.horizontalLayout_5.addWidget(self.sb_ctrl_channel)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem1)
        self.verticalLayout_5.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem2 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem2)
        self.dial_drywet = ScalableDial(self.groupBox_3)
        self.dial_drywet.setMinimumSize(QtCore.QSize(34, 34))
        self.dial_drywet.setMaximumSize(QtCore.QSize(34, 34))
        self.dial_drywet.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_drywet.setObjectName("dial_drywet")
        self.horizontalLayout_8.addWidget(self.dial_drywet)
        self.dial_vol = ScalableDial(self.groupBox_3)
        self.dial_vol.setMinimumSize(QtCore.QSize(34, 34))
        self.dial_vol.setMaximumSize(QtCore.QSize(34, 34))
        self.dial_vol.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_vol.setObjectName("dial_vol")
        self.horizontalLayout_8.addWidget(self.dial_vol)
        self.stackedWidget = QtWidgets.QStackedWidget(self.groupBox_3)
        self.stackedWidget.setMaximumSize(QtCore.QSize(16777215, 42))
        self.stackedWidget.setLineWidth(0)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page_bal = QtWidgets.QWidget()
        self.page_bal.setObjectName("page_bal")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.page_bal)
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_6.setSpacing(0)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.dial_b_left = ScalableDial(self.page_bal)
        self.dial_b_left.setMinimumSize(QtCore.QSize(26, 26))
        self.dial_b_left.setMaximumSize(QtCore.QSize(26, 26))
        self.dial_b_left.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_b_left.setObjectName("dial_b_left")
        self.horizontalLayout_6.addWidget(self.dial_b_left)
        self.dial_b_right = ScalableDial(self.page_bal)
        self.dial_b_right.setMinimumSize(QtCore.QSize(26, 26))
        self.dial_b_right.setMaximumSize(QtCore.QSize(26, 26))
        self.dial_b_right.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_b_right.setObjectName("dial_b_right")
        self.horizontalLayout_6.addWidget(self.dial_b_right)
        self.stackedWidget.addWidget(self.page_bal)
        self.page_pan = QtWidgets.QWidget()
        self.page_pan.setObjectName("page_pan")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.page_pan)
        self.horizontalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_7.setSpacing(0)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.dial_pan = ScalableDial(self.page_pan)
        self.dial_pan.setMinimumSize(QtCore.QSize(26, 26))
        self.dial_pan.setMaximumSize(QtCore.QSize(26, 26))
        self.dial_pan.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_pan.setObjectName("dial_pan")
        self.horizontalLayout_7.addWidget(self.dial_pan)
        self.stackedWidget.addWidget(self.page_pan)
        self.horizontalLayout_8.addWidget(self.stackedWidget)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.rb_balance = QtWidgets.QRadioButton(self.groupBox_3)
        self.rb_balance.setChecked(True)
        self.rb_balance.setObjectName("rb_balance")
        self.verticalLayout_2.addWidget(self.rb_balance)
        self.rb_pan = QtWidgets.QRadioButton(self.groupBox_3)
        self.rb_pan.setObjectName("rb_pan")
        self.verticalLayout_2.addWidget(self.rb_pan)
        self.horizontalLayout_8.addLayout(self.verticalLayout_2)
        spacerItem3 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem3)
        self.verticalLayout_5.addLayout(self.horizontalLayout_8)
        self.gridLayout_2.addWidget(self.groupBox_3, 2, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.tabEdit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.ch_use_chunks = QtWidgets.QCheckBox(self.groupBox)
        self.ch_use_chunks.setObjectName("ch_use_chunks")
        self.verticalLayout_4.addWidget(self.ch_use_chunks)
        self.line_2 = QtWidgets.QFrame(self.groupBox)
        self.line_2.setLineWidth(0)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_4.addWidget(self.line_2)
        self.label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout_4.addWidget(self.label)
        self.ch_fixed_buffer = QtWidgets.QCheckBox(self.groupBox)
        self.ch_fixed_buffer.setObjectName("ch_fixed_buffer")
        self.verticalLayout_4.addWidget(self.ch_fixed_buffer)
        self.ch_force_stereo = QtWidgets.QCheckBox(self.groupBox)
        self.ch_force_stereo.setObjectName("ch_force_stereo")
        self.verticalLayout_4.addWidget(self.ch_force_stereo)
        self.line = QtWidgets.QFrame(self.groupBox)
        self.line.setLineWidth(0)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_4.addWidget(self.line)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_4.addWidget(self.label_2)
        self.ch_map_program_changes = QtWidgets.QCheckBox(self.groupBox)
        self.ch_map_program_changes.setObjectName("ch_map_program_changes")
        self.verticalLayout_4.addWidget(self.ch_map_program_changes)
        self.ch_send_notes = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_notes.setObjectName("ch_send_notes")
        self.verticalLayout_4.addWidget(self.ch_send_notes)
        self.ch_send_program_changes = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_program_changes.setObjectName("ch_send_program_changes")
        self.verticalLayout_4.addWidget(self.ch_send_program_changes)
        self.ch_send_control_changes = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_control_changes.setObjectName("ch_send_control_changes")
        self.verticalLayout_4.addWidget(self.ch_send_control_changes)
        self.ch_send_channel_pressure = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_channel_pressure.setObjectName("ch_send_channel_pressure")
        self.verticalLayout_4.addWidget(self.ch_send_channel_pressure)
        self.ch_send_note_aftertouch = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_note_aftertouch.setObjectName("ch_send_note_aftertouch")
        self.verticalLayout_4.addWidget(self.ch_send_note_aftertouch)
        self.ch_send_pitchbend = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_pitchbend.setObjectName("ch_send_pitchbend")
        self.verticalLayout_4.addWidget(self.ch_send_pitchbend)
        self.ch_send_all_sound_off = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_all_sound_off.setObjectName("ch_send_all_sound_off")
        self.verticalLayout_4.addWidget(self.ch_send_all_sound_off)
        self.gridLayout_2.addWidget(self.groupBox, 1, 1, 2, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem4 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.label_plugin = QtWidgets.QLabel(self.tabEdit)
        self.label_plugin.setAlignment(QtCore.Qt.AlignCenter)
        self.label_plugin.setWordWrap(True)
        self.label_plugin.setObjectName("label_plugin")
        self.horizontalLayout.addWidget(self.label_plugin)
        spacerItem5 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem5)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.sw_programs = QtWidgets.QStackedWidget(self.tabEdit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sw_programs.sizePolicy().hasHeightForWidth())
        self.sw_programs.setSizePolicy(sizePolicy)
        self.sw_programs.setLineWidth(0)
        self.sw_programs.setObjectName("sw_programs")
        self.sww_programs = QtWidgets.QWidget()
        self.sww_programs.setObjectName("sww_programs")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.sww_programs)
        self.horizontalLayout_3.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem6 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem6)
        self.label_programs = QtWidgets.QLabel(self.sww_programs)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_programs.sizePolicy().hasHeightForWidth())
        self.label_programs.setSizePolicy(sizePolicy)
        self.label_programs.setObjectName("label_programs")
        self.horizontalLayout_3.addWidget(self.label_programs)
        self.cb_programs = QtWidgets.QComboBox(self.sww_programs)
        self.cb_programs.setMinimumSize(QtCore.QSize(150, 0))
        self.cb_programs.setObjectName("cb_programs")
        self.horizontalLayout_3.addWidget(self.cb_programs)
        self.sw_programs.addWidget(self.sww_programs)
        self.sww_midiPrograms = QtWidgets.QWidget()
        self.sww_midiPrograms.setObjectName("sww_midiPrograms")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.sww_midiPrograms)
        self.horizontalLayout_4.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem7 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem7)
        self.label_midi_programs = QtWidgets.QLabel(self.sww_midiPrograms)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_midi_programs.sizePolicy().hasHeightForWidth())
        self.label_midi_programs.setSizePolicy(sizePolicy)
        self.label_midi_programs.setObjectName("label_midi_programs")
        self.horizontalLayout_4.addWidget(self.label_midi_programs)
        self.cb_midi_programs = QtWidgets.QComboBox(self.sww_midiPrograms)
        self.cb_midi_programs.setMinimumSize(QtCore.QSize(150, 0))
        self.cb_midi_programs.setObjectName("cb_midi_programs")
        self.horizontalLayout_4.addWidget(self.cb_midi_programs)
        self.sw_programs.addWidget(self.sww_midiPrograms)
        self.gridLayout.addWidget(self.sw_programs, 0, 1, 1, 2)
        self.b_save_state = QtWidgets.QPushButton(self.tabEdit)
        self.b_save_state.setFocusPolicy(QtCore.Qt.NoFocus)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/document-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_save_state.setIcon(icon)
        self.b_save_state.setObjectName("b_save_state")
        self.gridLayout.addWidget(self.b_save_state, 1, 1, 1, 1)
        self.b_load_state = QtWidgets.QPushButton(self.tabEdit)
        self.b_load_state.setFocusPolicy(QtCore.Qt.NoFocus)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/16x16/document-open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_load_state.setIcon(icon1)
        self.b_load_state.setObjectName("b_load_state")
        self.gridLayout.addWidget(self.b_load_state, 1, 2, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem8, 1, 0, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout)
        self.gridLayout_2.addLayout(self.horizontalLayout, 0, 0, 1, 2)
        self.groupBox_2 = QtWidgets.QGroupBox(self.tabEdit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.le_maker = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_maker.setInputMask("")
        self.le_maker.setText("")
        self.le_maker.setFrame(False)
        self.le_maker.setReadOnly(True)
        self.le_maker.setObjectName("le_maker")
        self.gridLayout_3.addWidget(self.le_maker, 3, 1, 1, 1)
        self.label_label = QtWidgets.QLabel(self.groupBox_2)
        self.label_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_label.setObjectName("label_label")
        self.gridLayout_3.addWidget(self.label_label, 2, 0, 1, 1)
        self.label_name = QtWidgets.QLabel(self.groupBox_2)
        self.label_name.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_name.setObjectName("label_name")
        self.gridLayout_3.addWidget(self.label_name, 1, 0, 1, 1)
        self.le_name = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_name.setInputMask("")
        self.le_name.setText("")
        self.le_name.setFrame(False)
        self.le_name.setReadOnly(True)
        self.le_name.setObjectName("le_name")
        self.gridLayout_3.addWidget(self.le_name, 1, 1, 1, 1)
        self.le_type = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_type.setInputMask("")
        self.le_type.setText("")
        self.le_type.setFrame(False)
        self.le_type.setReadOnly(True)
        self.le_type.setObjectName("le_type")
        self.gridLayout_3.addWidget(self.le_type, 0, 1, 1, 1)
        self.le_copyright = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_copyright.setInputMask("")
        self.le_copyright.setText("")
        self.le_copyright.setFrame(False)
        self.le_copyright.setReadOnly(True)
        self.le_copyright.setObjectName("le_copyright")
        self.gridLayout_3.addWidget(self.le_copyright, 4, 1, 1, 1)
        self.le_unique_id = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_unique_id.setInputMask("")
        self.le_unique_id.setText("")
        self.le_unique_id.setFrame(False)
        self.le_unique_id.setReadOnly(True)
        self.le_unique_id.setObjectName("le_unique_id")
        self.gridLayout_3.addWidget(self.le_unique_id, 5, 1, 1, 1)
        self.label_type = QtWidgets.QLabel(self.groupBox_2)
        self.label_type.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_type.setObjectName("label_type")
        self.gridLayout_3.addWidget(self.label_type, 0, 0, 1, 1)
        self.label_maker = QtWidgets.QLabel(self.groupBox_2)
        self.label_maker.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_maker.setObjectName("label_maker")
        self.gridLayout_3.addWidget(self.label_maker, 3, 0, 1, 1)
        self.label_copyright = QtWidgets.QLabel(self.groupBox_2)
        self.label_copyright.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_copyright.setObjectName("label_copyright")
        self.gridLayout_3.addWidget(self.label_copyright, 4, 0, 1, 1)
        self.label_unique_id = QtWidgets.QLabel(self.groupBox_2)
        self.label_unique_id.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_unique_id.setObjectName("label_unique_id")
        self.gridLayout_3.addWidget(self.label_unique_id, 5, 0, 1, 1)
        self.le_label = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_label.setInputMask("")
        self.le_label.setText("")
        self.le_label.setFrame(False)
        self.le_label.setReadOnly(True)
        self.le_label.setObjectName("le_label")
        self.gridLayout_3.addWidget(self.le_label, 2, 1, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout_3)
        self.gridLayout_2.addWidget(self.groupBox_2, 1, 0, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        self.gridLayout_2.addItem(spacerItem9, 3, 0, 1, 2)
        self.tabWidget.addTab(self.tabEdit, "")
        self.verticalLayout.addWidget(self.tabWidget)

        self.retranslateUi(PluginEdit)
        self.tabWidget.setCurrentIndex(0)
        self.stackedWidget.setCurrentIndex(0)
        self.sw_programs.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(PluginEdit)

    def retranslateUi(self, PluginEdit):
        _translate = QtCore.QCoreApplication.translate
        PluginEdit.setWindowTitle(_translate("PluginEdit", "Plugin Editor"))
        self.groupBox_3.setTitle(_translate("PluginEdit", "Control"))
        self.label_ctrl_channel.setText(_translate("PluginEdit", "MIDI Control Channel:"))
        self.sb_ctrl_channel.setSpecialValueText(_translate("PluginEdit", "N"))
        self.dial_drywet.setStatusTip(_translate("PluginEdit", "Output dry/wet (100%)"))
        self.dial_vol.setStatusTip(_translate("PluginEdit", "Output volume (100%)"))
        self.dial_b_left.setStatusTip(_translate("PluginEdit", "Balance Left (0%)"))
        self.dial_b_right.setStatusTip(_translate("PluginEdit", "Balance Right (0%)"))
        self.dial_pan.setStatusTip(_translate("PluginEdit", "Balance Right (0%)"))
        self.rb_balance.setText(_translate("PluginEdit", "Use Balance"))
        self.rb_pan.setText(_translate("PluginEdit", "Use Panning"))
        self.groupBox.setTitle(_translate("PluginEdit", "Settings"))
        self.ch_use_chunks.setText(_translate("PluginEdit", "Use Chunks"))
        self.label.setText(_translate("PluginEdit", "    Audio:"))
        self.ch_fixed_buffer.setText(_translate("PluginEdit", "Fixed-Size Buffer"))
        self.ch_force_stereo.setText(_translate("PluginEdit", "Force Stereo (needs reload)"))
        self.label_2.setText(_translate("PluginEdit", "    MIDI:"))
        self.ch_map_program_changes.setText(_translate("PluginEdit", "Map Program Changes"))
        self.ch_send_notes.setText(_translate("PluginEdit", "Send Notes"))
        self.ch_send_program_changes.setText(_translate("PluginEdit", "Send Bank/Program Changes"))
        self.ch_send_control_changes.setText(_translate("PluginEdit", "Send Control Changes"))
        self.ch_send_channel_pressure.setText(_translate("PluginEdit", "Send Channel Pressure"))
        self.ch_send_note_aftertouch.setText(_translate("PluginEdit", "Send Note Aftertouch"))
        self.ch_send_pitchbend.setText(_translate("PluginEdit", "Send Pitchbend"))
        self.ch_send_all_sound_off.setText(_translate("PluginEdit", "Send All Sound/Notes Off"))
        self.label_plugin.setText(_translate("PluginEdit", "\n"
"Plugin Name\n"
""))
        self.label_programs.setText(_translate("PluginEdit", "Program:"))
        self.label_midi_programs.setText(_translate("PluginEdit", "MIDI Program:"))
        self.b_save_state.setText(_translate("PluginEdit", "Save State"))
        self.b_load_state.setText(_translate("PluginEdit", "Load State"))
        self.groupBox_2.setTitle(_translate("PluginEdit", "Information"))
        self.label_label.setText(_translate("PluginEdit", "Label/URI:"))
        self.label_name.setText(_translate("PluginEdit", "Name:"))
        self.label_type.setText(_translate("PluginEdit", "Type:"))
        self.label_maker.setText(_translate("PluginEdit", "Maker:"))
        self.label_copyright.setText(_translate("PluginEdit", "Copyright:"))
        self.label_unique_id.setText(_translate("PluginEdit", "Unique ID:"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabEdit), _translate("PluginEdit", "Edit"))
from widgets.scalabledial import ScalableDial
import resources_rc
