# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_plugin_presets.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginWidget(object):
    def setupUi(self, PluginWidget):
        PluginWidget.setObjectName("PluginWidget")
        PluginWidget.resize(484, 65)
        PluginWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.verticalLayout = QtWidgets.QVBoxLayout(PluginWidget)
        self.verticalLayout.setContentsMargins(4, 3, 6, 3)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.layout_top = QtWidgets.QHBoxLayout()
        self.layout_top.setSpacing(1)
        self.layout_top.setObjectName("layout_top")
        self.w_buttons = QtWidgets.QWidget(PluginWidget)
        self.w_buttons.setMinimumSize(QtCore.QSize(72, 24))
        self.w_buttons.setMaximumSize(QtCore.QSize(72, 24))
        self.w_buttons.setObjectName("w_buttons")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.w_buttons)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.b_enable = ScalableButton(self.w_buttons)
        self.b_enable.setMinimumSize(QtCore.QSize(24, 24))
        self.b_enable.setMaximumSize(QtCore.QSize(24, 24))
        self.b_enable.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/bitmaps/button_off.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_enable.setIcon(icon)
        self.b_enable.setIconSize(QtCore.QSize(24, 24))
        self.b_enable.setCheckable(True)
        self.b_enable.setFlat(True)
        self.b_enable.setObjectName("b_enable")
        self.horizontalLayout.addWidget(self.b_enable)
        self.b_gui = ScalableButton(self.w_buttons)
        self.b_gui.setMinimumSize(QtCore.QSize(24, 24))
        self.b_gui.setMaximumSize(QtCore.QSize(24, 24))
        self.b_gui.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/bitmaps/button_gui.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_gui.setIcon(icon1)
        self.b_gui.setIconSize(QtCore.QSize(24, 24))
        self.b_gui.setCheckable(True)
        self.b_gui.setFlat(True)
        self.b_gui.setObjectName("b_gui")
        self.horizontalLayout.addWidget(self.b_gui)
        self.b_edit = ScalableButton(self.w_buttons)
        self.b_edit.setMinimumSize(QtCore.QSize(24, 24))
        self.b_edit.setMaximumSize(QtCore.QSize(24, 24))
        self.b_edit.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/bitmaps/button_edit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_edit.setIcon(icon2)
        self.b_edit.setIconSize(QtCore.QSize(24, 24))
        self.b_edit.setCheckable(True)
        self.b_edit.setFlat(True)
        self.b_edit.setObjectName("b_edit")
        self.horizontalLayout.addWidget(self.b_edit)
        self.layout_top.addWidget(self.w_buttons)
        spacerItem = QtWidgets.QSpacerItem(6, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.layout_top.addItem(spacerItem)
        self.line_2 = QtWidgets.QFrame(PluginWidget)
        self.line_2.setMinimumSize(QtCore.QSize(0, 20))
        self.line_2.setMaximumSize(QtCore.QSize(16777215, 20))
        self.line_2.setLineWidth(0)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.layout_top.addWidget(self.line_2)
        spacerItem1 = QtWidgets.QSpacerItem(8, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.layout_top.addItem(spacerItem1)
        self.label_name = QtWidgets.QLabel(PluginWidget)
        self.label_name.setTextFormat(QtCore.Qt.PlainText)
        self.label_name.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_name.setObjectName("label_name")
        self.layout_top.addWidget(self.label_name)
        spacerItem2 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_top.addItem(spacerItem2)
        self.layout_presets = QtWidgets.QHBoxLayout()
        self.layout_presets.setContentsMargins(4, -1, 4, -1)
        self.layout_presets.setSpacing(4)
        self.layout_presets.setObjectName("layout_presets")
        self.label_presets = QtWidgets.QLabel(PluginWidget)
        self.label_presets.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_presets.setObjectName("label_presets")
        self.layout_presets.addWidget(self.label_presets)
        self.cb_presets = QtWidgets.QComboBox(PluginWidget)
        self.cb_presets.setMinimumSize(QtCore.QSize(125, 0))
        self.cb_presets.setObjectName("cb_presets")
        self.layout_presets.addWidget(self.cb_presets)
        self.layout_top.addLayout(self.layout_presets)
        self.layout_leds = QtWidgets.QHBoxLayout()
        self.layout_leds.setSpacing(1)
        self.layout_leds.setObjectName("layout_leds")
        self.led_control = LEDButton(PluginWidget)
        self.led_control.setMinimumSize(QtCore.QSize(14, 14))
        self.led_control.setMaximumSize(QtCore.QSize(14, 14))
        self.led_control.setText("")
        self.led_control.setCheckable(True)
        self.led_control.setObjectName("led_control")
        self.layout_leds.addWidget(self.led_control)
        self.led_midi = LEDButton(PluginWidget)
        self.led_midi.setMinimumSize(QtCore.QSize(14, 14))
        self.led_midi.setMaximumSize(QtCore.QSize(14, 14))
        self.led_midi.setText("")
        self.led_midi.setCheckable(True)
        self.led_midi.setObjectName("led_midi")
        self.layout_leds.addWidget(self.led_midi)
        self.led_audio_in = LEDButton(PluginWidget)
        self.led_audio_in.setMinimumSize(QtCore.QSize(14, 14))
        self.led_audio_in.setMaximumSize(QtCore.QSize(14, 14))
        self.led_audio_in.setText("")
        self.led_audio_in.setCheckable(True)
        self.led_audio_in.setObjectName("led_audio_in")
        self.layout_leds.addWidget(self.led_audio_in)
        self.led_audio_out = LEDButton(PluginWidget)
        self.led_audio_out.setMinimumSize(QtCore.QSize(14, 14))
        self.led_audio_out.setMaximumSize(QtCore.QSize(14, 14))
        self.led_audio_out.setText("")
        self.led_audio_out.setCheckable(True)
        self.led_audio_out.setObjectName("led_audio_out")
        self.layout_leds.addWidget(self.led_audio_out)
        self.layout_top.addLayout(self.layout_leds)
        self.verticalLayout.addLayout(self.layout_top)
        self.line = QtWidgets.QFrame(PluginWidget)
        self.line.setLineWidth(0)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.layout_bottom = QtWidgets.QHBoxLayout()
        self.layout_bottom.setContentsMargins(12, 4, 12, -1)
        self.layout_bottom.setSpacing(1)
        self.layout_bottom.setObjectName("layout_bottom")
        self.w_knobs_left = QtWidgets.QWidget(PluginWidget)
        self.w_knobs_left.setObjectName("w_knobs_left")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.w_knobs_left)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.layout_bottom.addWidget(self.w_knobs_left)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_bottom.addItem(spacerItem3)
        self.w_knobs_right = QtWidgets.QWidget(PluginWidget)
        self.w_knobs_right.setObjectName("w_knobs_right")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.w_knobs_right)
        self.horizontalLayout_3.setContentsMargins(12, 0, 0, 0)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.layout_bottom.addWidget(self.w_knobs_right)
        self.layout_peaks = QtWidgets.QVBoxLayout()
        self.layout_peaks.setContentsMargins(4, 0, 4, 2)
        self.layout_peaks.setSpacing(4)
        self.layout_peaks.setObjectName("layout_peaks")
        self.peak_in = DigitalPeakMeter(PluginWidget)
        self.peak_in.setMinimumSize(QtCore.QSize(150, 0))
        self.peak_in.setMaximumSize(QtCore.QSize(150, 16777215))
        self.peak_in.setObjectName("peak_in")
        self.layout_peaks.addWidget(self.peak_in)
        self.peak_out = DigitalPeakMeter(PluginWidget)
        self.peak_out.setMinimumSize(QtCore.QSize(150, 0))
        self.peak_out.setMaximumSize(QtCore.QSize(150, 16777215))
        self.peak_out.setObjectName("peak_out")
        self.layout_peaks.addWidget(self.peak_out)
        self.layout_bottom.addLayout(self.layout_peaks)
        self.verticalLayout.addLayout(self.layout_bottom)

        self.retranslateUi(PluginWidget)
        QtCore.QMetaObject.connectSlotsByName(PluginWidget)

    def retranslateUi(self, PluginWidget):
        _translate = QtCore.QCoreApplication.translate
        PluginWidget.setWindowTitle(_translate("PluginWidget", "Frame"))
        self.label_name.setText(_translate("PluginWidget", "PluginName"))
        self.label_presets.setText(_translate("PluginWidget", "Preset:"))
from widgets.digitalpeakmeter import DigitalPeakMeter
from widgets.ledbutton import LEDButton
from widgets.scalablebutton import ScalableButton
import resources_rc
