# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_add_jack.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(458, 546)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.l_note_features = QtWidgets.QLabel(Dialog)
        self.l_note_features.setObjectName("l_note_features")
        self.verticalLayout.addWidget(self.l_note_features)
        self.group_command = QtWidgets.QGroupBox(Dialog)
        self.group_command.setObjectName("group_command")
        self.gridLayout = QtWidgets.QGridLayout(self.group_command)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem = QtWidgets.QSpacerItem(20, 60, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 0, 3, 1)
        self.label_10 = QtWidgets.QLabel(self.group_command)
        self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_10.setObjectName("label_10")
        self.gridLayout.addWidget(self.label_10, 0, 1, 1, 1)
        self.le_name = QtWidgets.QLineEdit(self.group_command)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_name.sizePolicy().hasHeightForWidth())
        self.le_name.setSizePolicy(sizePolicy)
        self.le_name.setObjectName("le_name")
        self.gridLayout.addWidget(self.le_name, 0, 2, 1, 2)
        spacerItem1 = QtWidgets.QSpacerItem(20, 60, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 0, 4, 3, 1)
        self.label_5 = QtWidgets.QLabel(self.group_command)
        self.label_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 1, 1, 1, 1)
        self.rb_template = QtWidgets.QRadioButton(self.group_command)
        self.rb_template.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.rb_template.sizePolicy().hasHeightForWidth())
        self.rb_template.setSizePolicy(sizePolicy)
        self.rb_template.setObjectName("rb_template")
        self.gridLayout.addWidget(self.rb_template, 1, 2, 1, 1)
        self.rb_custom = QtWidgets.QRadioButton(self.group_command)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.rb_custom.sizePolicy().hasHeightForWidth())
        self.rb_custom.setSizePolicy(sizePolicy)
        self.rb_custom.setChecked(True)
        self.rb_custom.setObjectName("rb_custom")
        self.gridLayout.addWidget(self.rb_custom, 1, 3, 1, 1)
        self.stackedWidget = QtWidgets.QStackedWidget(self.group_command)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page_template = QtWidgets.QWidget()
        self.page_template.setObjectName("page_template")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.page_template)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.l_template = QtWidgets.QLabel(self.page_template)
        self.l_template.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.l_template.setObjectName("l_template")
        self.horizontalLayout_2.addWidget(self.l_template)
        self.cb_template = QtWidgets.QComboBox(self.page_template)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cb_template.sizePolicy().hasHeightForWidth())
        self.cb_template.setSizePolicy(sizePolicy)
        self.cb_template.setEditable(False)
        self.cb_template.setObjectName("cb_template")
        self.horizontalLayout_2.addWidget(self.cb_template)
        self.stackedWidget.addWidget(self.page_template)
        self.page_command = QtWidgets.QWidget()
        self.page_command.setObjectName("page_command")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.page_command)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.l_command = QtWidgets.QLabel(self.page_command)
        self.l_command.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.l_command.setObjectName("l_command")
        self.horizontalLayout_3.addWidget(self.l_command)
        self.le_command = QtWidgets.QLineEdit(self.page_command)
        self.le_command.setObjectName("le_command")
        self.horizontalLayout_3.addWidget(self.le_command)
        self.stackedWidget.addWidget(self.page_command)
        self.gridLayout.addWidget(self.stackedWidget, 2, 1, 1, 3)
        self.verticalLayout.addWidget(self.group_command)
        self.group_setup = QtWidgets.QGroupBox(Dialog)
        self.group_setup.setObjectName("group_setup")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.group_setup)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem2, 2, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.label_6 = QtWidgets.QLabel(self.group_setup)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout.addWidget(self.label_6)
        self.cb_session_mgr = QtWidgets.QComboBox(self.group_setup)
        self.cb_session_mgr.setObjectName("cb_session_mgr")
        self.cb_session_mgr.addItem("")
        self.cb_session_mgr.addItem("")
        self.cb_session_mgr.setItemText(1, "LADISH (SIGUSR1)")
        self.cb_session_mgr.addItem("")
        self.cb_session_mgr.setItemText(2, "NSM")
        self.horizontalLayout.addWidget(self.cb_session_mgr)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.gridLayout_2.addLayout(self.horizontalLayout, 0, 1, 1, 8)
        self.label_3 = QtWidgets.QLabel(self.group_setup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 2, 1, 1, 1)
        self.sb_audio_ins = QtWidgets.QSpinBox(self.group_setup)
        self.sb_audio_ins.setMaximum(64)
        self.sb_audio_ins.setObjectName("sb_audio_ins")
        self.gridLayout_2.addWidget(self.sb_audio_ins, 2, 2, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem5, 2, 3, 1, 1)
        self.line = QtWidgets.QFrame(self.group_setup)
        self.line.setLineWidth(0)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_2.addWidget(self.line, 2, 4, 2, 1)
        spacerItem6 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem6, 2, 5, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.group_setup)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 2, 6, 1, 1)
        self.sb_midi_ins = QtWidgets.QSpinBox(self.group_setup)
        self.sb_midi_ins.setMaximum(1)
        self.sb_midi_ins.setObjectName("sb_midi_ins")
        self.gridLayout_2.addWidget(self.sb_midi_ins, 2, 7, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem7, 2, 8, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem8, 3, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.group_setup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 3, 1, 1, 1)
        self.sb_audio_outs = QtWidgets.QSpinBox(self.group_setup)
        self.sb_audio_outs.setMaximum(64)
        self.sb_audio_outs.setObjectName("sb_audio_outs")
        self.gridLayout_2.addWidget(self.sb_audio_outs, 3, 2, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem9, 3, 3, 1, 1)
        spacerItem10 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem10, 3, 5, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.group_setup)
        self.label_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_8.setObjectName("label_8")
        self.gridLayout_2.addWidget(self.label_8, 3, 6, 1, 1)
        self.sb_midi_outs = QtWidgets.QSpinBox(self.group_setup)
        self.sb_midi_outs.setMaximum(1)
        self.sb_midi_outs.setObjectName("sb_midi_outs")
        self.gridLayout_2.addWidget(self.sb_midi_outs, 3, 7, 1, 1)
        spacerItem11 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem11, 3, 8, 1, 1)
        self.cb_manage_window = QtWidgets.QCheckBox(self.group_setup)
        self.cb_manage_window.setObjectName("cb_manage_window")
        self.gridLayout_2.addWidget(self.cb_manage_window, 1, 2, 1, 7)
        self.verticalLayout.addWidget(self.group_setup)
        self.group_workarounds = QtWidgets.QGroupBox(Dialog)
        self.group_workarounds.setObjectName("group_workarounds")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.group_workarounds)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.cb_external_start = QtWidgets.QCheckBox(self.group_workarounds)
        self.cb_external_start.setObjectName("cb_external_start")
        self.gridLayout_3.addWidget(self.cb_external_start, 4, 1, 1, 1)
        self.cb_capture_first_window = QtWidgets.QCheckBox(self.group_workarounds)
        self.cb_capture_first_window.setEnabled(False)
        self.cb_capture_first_window.setObjectName("cb_capture_first_window")
        self.gridLayout_3.addWidget(self.cb_capture_first_window, 0, 1, 1, 1)
        spacerItem12 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem12, 0, 0, 1, 1)
        spacerItem13 = QtWidgets.QSpacerItem(135, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem13, 0, 2, 1, 1)
        self.cb_buffers_addition_mode = QtWidgets.QCheckBox(self.group_workarounds)
        self.cb_buffers_addition_mode.setObjectName("cb_buffers_addition_mode")
        self.gridLayout_3.addWidget(self.cb_buffers_addition_mode, 2, 1, 1, 1)
        self.cb_out_midi_mixdown = QtWidgets.QCheckBox(self.group_workarounds)
        self.cb_out_midi_mixdown.setObjectName("cb_out_midi_mixdown")
        self.gridLayout_3.addWidget(self.cb_out_midi_mixdown, 1, 1, 1, 1)
        self.verticalLayout.addWidget(self.group_workarounds)
        self.group_error = QtWidgets.QFrame(Dialog)
        self.group_error.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.group_error.setFrameShadow(QtWidgets.QFrame.Raised)
        self.group_error.setObjectName("group_error")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.group_error)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.ico_error = QtWidgets.QLabel(self.group_error)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(16)
        sizePolicy.setVerticalStretch(16)
        sizePolicy.setHeightForWidth(self.ico_error.sizePolicy().hasHeightForWidth())
        self.ico_error.setSizePolicy(sizePolicy)
        self.ico_error.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_error.setText("")
        self.ico_error.setPixmap(QtGui.QPixmap(":/16x16/dialog-cancel.svgz"))
        self.ico_error.setScaledContents(True)
        self.ico_error.setObjectName("ico_error")
        self.horizontalLayout_4.addWidget(self.ico_error)
        self.l_error = QtWidgets.QLabel(self.group_error)
        self.l_error.setObjectName("l_error")
        self.horizontalLayout_4.addWidget(self.l_error)
        self.verticalLayout.addWidget(self.group_error)
        spacerItem14 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem14)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.stackedWidget.setCurrentIndex(1)
        self.cb_session_mgr.setCurrentIndex(0)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Add JACK Application"))
        self.l_note_features.setText(_translate("Dialog", "Note: Features not implemented yet are greyed out"))
        self.group_command.setTitle(_translate("Dialog", "Application"))
        self.label_10.setText(_translate("Dialog", "Name:"))
        self.label_5.setText(_translate("Dialog", "Application:"))
        self.rb_template.setText(_translate("Dialog", "From template"))
        self.rb_custom.setText(_translate("Dialog", "Custom"))
        self.l_template.setText(_translate("Dialog", "Template:"))
        self.l_command.setText(_translate("Dialog", "Command:"))
        self.group_setup.setTitle(_translate("Dialog", "Setup"))
        self.label_6.setText(_translate("Dialog", "Session Manager:"))
        self.cb_session_mgr.setItemText(0, _translate("Dialog", "None"))
        self.label_3.setText(_translate("Dialog", "Audio inputs:"))
        self.label_7.setText(_translate("Dialog", "MIDI inputs:"))
        self.label_4.setText(_translate("Dialog", "Audio outputs:"))
        self.label_8.setText(_translate("Dialog", "MIDI outputs:"))
        self.cb_manage_window.setText(_translate("Dialog", "Take control of main application window"))
        self.group_workarounds.setTitle(_translate("Dialog", "Workarounds"))
        self.cb_external_start.setText(_translate("Dialog", "Wait for external application start (Advanced, for Debug only)"))
        self.cb_capture_first_window.setText(_translate("Dialog", "Capture only the first X11 Window"))
        self.cb_buffers_addition_mode.setText(_translate("Dialog", "Use previous client output buffer as input for the next client"))
        self.cb_out_midi_mixdown.setText(_translate("Dialog", "Simulate 16 JACK MIDI outputs, with MIDI channel as port index"))
        self.l_error.setText(_translate("Dialog", "Error here"))
import resources_rc
