# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/midipattern.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MidiPatternW(object):
    def setupUi(self, MidiPatternW):
        MidiPatternW.setObjectName("MidiPatternW")
        MidiPatternW.resize(755, 436)
        self.centralwidget = QtWidgets.QWidget(MidiPatternW)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.modeIndicator = ModeIndicator(self.centralwidget)
        self.modeIndicator.setMinimumSize(QtCore.QSize(30, 20))
        self.modeIndicator.setMaximumSize(QtCore.QSize(30, 20))
        self.modeIndicator.setObjectName("modeIndicator")
        self.horizontalLayout.addWidget(self.modeIndicator)
        self.timeSigLabel = QtWidgets.QLabel(self.centralwidget)
        self.timeSigLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeSigLabel.setObjectName("timeSigLabel")
        self.horizontalLayout.addWidget(self.timeSigLabel)
        self.timeSigBox = QtWidgets.QComboBox(self.centralwidget)
        self.timeSigBox.setEditable(True)
        self.timeSigBox.setObjectName("timeSigBox")
        self.timeSigBox.addItem("")
        self.timeSigBox.addItem("")
        self.timeSigBox.addItem("")
        self.timeSigBox.addItem("")
        self.timeSigBox.addItem("")
        self.timeSigBox.addItem("")
        self.horizontalLayout.addWidget(self.timeSigBox)
        spacerItem = QtWidgets.QSpacerItem(5, 5, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.measureLabel = QtWidgets.QLabel(self.centralwidget)
        self.measureLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.measureLabel.setObjectName("measureLabel")
        self.horizontalLayout.addWidget(self.measureLabel)
        self.measureBox = QtWidgets.QComboBox(self.centralwidget)
        self.measureBox.setObjectName("measureBox")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.measureBox.addItem("")
        self.horizontalLayout.addWidget(self.measureBox)
        spacerItem1 = QtWidgets.QSpacerItem(5, 5, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.defaultLengthLabel = QtWidgets.QLabel(self.centralwidget)
        self.defaultLengthLabel.setObjectName("defaultLengthLabel")
        self.horizontalLayout.addWidget(self.defaultLengthLabel)
        self.defaultLengthBox = QtWidgets.QComboBox(self.centralwidget)
        self.defaultLengthBox.setObjectName("defaultLengthBox")
        self.defaultLengthBox.addItem("")
        self.defaultLengthBox.addItem("")
        self.defaultLengthBox.addItem("")
        self.defaultLengthBox.addItem("")
        self.defaultLengthBox.addItem("")
        self.defaultLengthBox.addItem("")
        self.defaultLengthBox.addItem("")
        self.defaultLengthBox.addItem("")
        self.defaultLengthBox.addItem("")
        self.defaultLengthBox.addItem("")
        self.horizontalLayout.addWidget(self.defaultLengthBox)
        spacerItem2 = QtWidgets.QSpacerItem(5, 5, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.quantizeLabel = QtWidgets.QLabel(self.centralwidget)
        self.quantizeLabel.setObjectName("quantizeLabel")
        self.horizontalLayout.addWidget(self.quantizeLabel)
        self.quantizeBox = QtWidgets.QComboBox(self.centralwidget)
        self.quantizeBox.setObjectName("quantizeBox")
        self.quantizeBox.addItem("")
        self.quantizeBox.addItem("")
        self.quantizeBox.addItem("")
        self.quantizeBox.addItem("")
        self.quantizeBox.addItem("")
        self.quantizeBox.addItem("")
        self.quantizeBox.addItem("")
        self.quantizeBox.addItem("")
        self.quantizeBox.addItem("")
        self.quantizeBox.addItem("")
        self.horizontalLayout.addWidget(self.quantizeBox)
        spacerItem3 = QtWidgets.QSpacerItem(5, 5, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.hSlider = QtWidgets.QSlider(self.centralwidget)
        self.hSlider.setOrientation(QtCore.Qt.Horizontal)
        self.hSlider.setObjectName("hSlider")
        self.horizontalLayout.addWidget(self.hSlider)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.vSlider = QtWidgets.QSlider(self.centralwidget)
        self.vSlider.setOrientation(QtCore.Qt.Vertical)
        self.vSlider.setObjectName("vSlider")
        self.horizontalLayout_2.addWidget(self.vSlider)
        self.graphicsView = PianoRollView(self.centralwidget)
        self.graphicsView.setObjectName("graphicsView")
        self.horizontalLayout_2.addWidget(self.graphicsView)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        MidiPatternW.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MidiPatternW)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 755, 23))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Edit = QtWidgets.QMenu(self.menubar)
        self.menu_Edit.setObjectName("menu_Edit")
        MidiPatternW.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MidiPatternW)
        self.statusbar.setObjectName("statusbar")
        MidiPatternW.setStatusBar(self.statusbar)
        self.act_file_quit = QtWidgets.QAction(MidiPatternW)
        self.act_file_quit.setObjectName("act_file_quit")
        self.act_edit_insert = QtWidgets.QAction(MidiPatternW)
        self.act_edit_insert.setCheckable(True)
        self.act_edit_insert.setObjectName("act_edit_insert")
        self.act_edit_velocity = QtWidgets.QAction(MidiPatternW)
        self.act_edit_velocity.setCheckable(True)
        self.act_edit_velocity.setObjectName("act_edit_velocity")
        self.act_edit_select_all = QtWidgets.QAction(MidiPatternW)
        self.act_edit_select_all.setObjectName("act_edit_select_all")
        self.menu_File.addAction(self.act_file_quit)
        self.menu_Edit.addAction(self.act_edit_insert)
        self.menu_Edit.addAction(self.act_edit_velocity)
        self.menu_Edit.addAction(self.act_edit_select_all)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Edit.menuAction())

        self.retranslateUi(MidiPatternW)
        self.timeSigBox.setCurrentIndex(0)
        self.measureBox.setCurrentIndex(3)
        self.defaultLengthBox.setCurrentIndex(4)
        self.quantizeBox.setCurrentIndex(4)
        self.act_file_quit.triggered.connect(MidiPatternW.close) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MidiPatternW)

    def retranslateUi(self, MidiPatternW):
        _translate = QtCore.QCoreApplication.translate
        MidiPatternW.setWindowTitle(_translate("MidiPatternW", "MIDI Pattern"))
        self.timeSigLabel.setText(_translate("MidiPatternW", "Time Signature:"))
        self.timeSigBox.setItemText(0, _translate("MidiPatternW", "1/4"))
        self.timeSigBox.setItemText(1, _translate("MidiPatternW", "2/4"))
        self.timeSigBox.setItemText(2, _translate("MidiPatternW", "3/4"))
        self.timeSigBox.setItemText(3, _translate("MidiPatternW", "4/4"))
        self.timeSigBox.setItemText(4, _translate("MidiPatternW", "5/4"))
        self.timeSigBox.setItemText(5, _translate("MidiPatternW", "6/4"))
        self.measureLabel.setText(_translate("MidiPatternW", "Measures:"))
        self.measureBox.setItemText(0, _translate("MidiPatternW", "1"))
        self.measureBox.setItemText(1, _translate("MidiPatternW", "2"))
        self.measureBox.setItemText(2, _translate("MidiPatternW", "3"))
        self.measureBox.setItemText(3, _translate("MidiPatternW", "4"))
        self.measureBox.setItemText(4, _translate("MidiPatternW", "5"))
        self.measureBox.setItemText(5, _translate("MidiPatternW", "6"))
        self.measureBox.setItemText(6, _translate("MidiPatternW", "7"))
        self.measureBox.setItemText(7, _translate("MidiPatternW", "8"))
        self.measureBox.setItemText(8, _translate("MidiPatternW", "9"))
        self.measureBox.setItemText(9, _translate("MidiPatternW", "10"))
        self.measureBox.setItemText(10, _translate("MidiPatternW", "11"))
        self.measureBox.setItemText(11, _translate("MidiPatternW", "12"))
        self.measureBox.setItemText(12, _translate("MidiPatternW", "13"))
        self.measureBox.setItemText(13, _translate("MidiPatternW", "14"))
        self.measureBox.setItemText(14, _translate("MidiPatternW", "15"))
        self.measureBox.setItemText(15, _translate("MidiPatternW", "16"))
        self.defaultLengthLabel.setText(_translate("MidiPatternW", "Default Length:"))
        self.defaultLengthBox.setItemText(0, _translate("MidiPatternW", "1/16"))
        self.defaultLengthBox.setItemText(1, _translate("MidiPatternW", "1/15"))
        self.defaultLengthBox.setItemText(2, _translate("MidiPatternW", "1/12"))
        self.defaultLengthBox.setItemText(3, _translate("MidiPatternW", "1/9"))
        self.defaultLengthBox.setItemText(4, _translate("MidiPatternW", "1/8"))
        self.defaultLengthBox.setItemText(5, _translate("MidiPatternW", "1/6"))
        self.defaultLengthBox.setItemText(6, _translate("MidiPatternW", "1/4"))
        self.defaultLengthBox.setItemText(7, _translate("MidiPatternW", "1/3"))
        self.defaultLengthBox.setItemText(8, _translate("MidiPatternW", "1/2"))
        self.defaultLengthBox.setItemText(9, _translate("MidiPatternW", "1"))
        self.quantizeLabel.setText(_translate("MidiPatternW", "Quantize:"))
        self.quantizeBox.setItemText(0, _translate("MidiPatternW", "1/16"))
        self.quantizeBox.setItemText(1, _translate("MidiPatternW", "1/15"))
        self.quantizeBox.setItemText(2, _translate("MidiPatternW", "1/12"))
        self.quantizeBox.setItemText(3, _translate("MidiPatternW", "1/9"))
        self.quantizeBox.setItemText(4, _translate("MidiPatternW", "1/8"))
        self.quantizeBox.setItemText(5, _translate("MidiPatternW", "1/6"))
        self.quantizeBox.setItemText(6, _translate("MidiPatternW", "1/4"))
        self.quantizeBox.setItemText(7, _translate("MidiPatternW", "1/3"))
        self.quantizeBox.setItemText(8, _translate("MidiPatternW", "1/2"))
        self.quantizeBox.setItemText(9, _translate("MidiPatternW", "1"))
        self.menu_File.setTitle(_translate("MidiPatternW", "&File"))
        self.menu_Edit.setTitle(_translate("MidiPatternW", "&Edit"))
        self.act_file_quit.setText(_translate("MidiPatternW", "&Quit"))
        self.act_file_quit.setShortcut(_translate("MidiPatternW", "Esc"))
        self.act_edit_insert.setText(_translate("MidiPatternW", "&Insert Mode"))
        self.act_edit_insert.setShortcut(_translate("MidiPatternW", "F"))
        self.act_edit_velocity.setText(_translate("MidiPatternW", "&Velocity Mode"))
        self.act_edit_velocity.setShortcut(_translate("MidiPatternW", "D"))
        self.act_edit_select_all.setText(_translate("MidiPatternW", "Select All"))
        self.act_edit_select_all.setShortcut(_translate("MidiPatternW", "A"))
from widgets.pianoroll import ModeIndicator, PianoRollView
